
-----
Page Url: src/experimental/003 - Graph Tests.md
-----

# Example Graph Tests

## TEST Mermaid
 
```mermaid
graph LR
	users["Users"]  
	subgraph "Users' computers"
		webBrowser["Web Browser"] 
	end
	subgraph "Web Server"
		webApp["Web Application"]
	end
	subgraph "Database Server"
		db["Database"]
	end
	users -- Use --> webBrowser 
	webBrowser -- HTTP --> webApp
	webApp -- SQL --> db
```

```mermaid
journey
  title My working day 20
  section Go to work 
    Make tea: 5: Me 
    Go upstairs: 3: Me
    Do work: 3: Me, Cat
  section Go home
    Sit down: 5: Her
```-----
Page Url: src/experimental/README.md
-----
Just some test examples-----
Page Url: src/experimental/002 - Test Code examples.md
-----
# Code Highlighting

```javascript
const a = 3
```

```typescript
const a: number = 3
```

```markdown
## TEST
```

```html
<ae-design name="test">
</ae-design>
```

```typescript{3-5}
/* global URL, process */
import path from 'path'

/* eslint-disable-next-line */
export function dirname(importMeta: any): string {
  /* c8 ignore next 3 */
  const d = path.dirname(decodeURI(new URL(importMeta.url).pathname))
  return path.resolve((process.platform == "win32") ? d.substr(1) : d) // eslint-disable-line 2580
}

```

<!-- specify the code language -->
@[code{4-9} ts{4-5}](../../config/navbar.ts)

```markdown
highlight line 3 of the imported code, i.e. line 5 of the '../foo.js' file
disable line numbers
@[code{3-10} js{3}:no-line-numbers](../foo.js)
Notice that path aliases are not available in import code syntax. You can use following config to handle path alias yourself:

module.exports = {
  markdown: {
    importCode: {
      handleImportPath: (str) =>
        str.replace(/^@src/, path.resolve(__dirname, 'path/to/src')),
    },
  },
}
<!-- it will be resolved to 'path/to/src/foo.js' -->
@[code](@src/foo.js)
::: tip This import code extension is supported by our built-in plugin.

```-----
Page Url: src/contributing.md
-----
-----
Page Url: src/guide/001 - Getting Started.md
-----
<div class="flex flex-col items-center mb-10">
  <img width="20%" src="/images/assets-envato/cute-programmer-working-on-laptop-cartoon-2021-12-09-18-09-14-utc/600_84.svg">
  <pre style="margin:0;padding:0">...This documentation is still work in progress 😇 ...</pre>
</div>

<div class="inline-flex whitespace-nowrap text-xs font-bold rounded-sm shadow overflow-hidden mb-4">
  <span class="text-gray-100 p-1 bg-sky-700">version</span>
  <version class="bg-sky-800 text-white p-1"></version>
</div>


# Getting Started

Install @aeppic/server with default settings and no-data using: 

`npx @aeppic/install-server`

Update base data to use only:

`npx @aeppic/install-server --data-version=latest --data-only`

> More about install-server can be found in tools-----
Page Url: src/about/README.md
-----
<script setup>
import { VPTeamMembers } from 'vitepress/theme'

const members = [
  {
    avatar: 'https://www.github.com/matthiasg.png',
    name: 'Matthias Götzke',
    title: 'Creator',
    links: [
      { icon: 'github', link: 'https://github.com/matthiasg' },
      { icon: 'twitter', link: 'https://twitter.com/mgoetzke' }
    ]
  }
]
</script>

# About

**aeppic** was created and is owned by **curasystems GmbH** Germany based on years of experience developing medical and non-medical custom software solutions.

The main inventor and creator is Matthias Götzke who is also Co-Founder of curasystems and its Lead Developer alongside help from internal and external developers over the years. 

<VPTeamMembers size="medium" :members="members" />-----
Page Url: src/contributing/README.md
-----
# Overview

Contributions are currently only possible by curasystems GmbH.

## Build process

The build process is handled via `@microsoft/rush`:

Once all dependencies are linked and the environment matches the following command
does the build:

`node common/scripts/install-run-rush.js build`

During development the following command is more helpful:

`node common/scripts/install-run-rush.js build:incremental`

and if `npm i -g @microsoft/rush` was installed before you can shorten that to:

`rush build:incremental`

### Dependencies 

The `.gitlab-ci.yml` file shows all the other steps that need to be done initially too.

<!-- specify the code language -->
<<< @/../../.gitlab-ci.yml{103-11 yaml:line-numbers}

### Environment

In order to have repeatable builds in the CI environment we use a docker image which is declared in

`/ci/Dockerfile`

and built with `build.sh` next to it. The `build.sh` requires a correct version number inside it and the
`.gitlab-ci.yml` file also needs to reference it. It can only be referenced if it has also been pushed to
the build server before of course.
-----
Page Url: src/reference/aeppic-service-worker/README.md
-----
# @aeppic/service-worker

The service worker project compiles into a javascript file which can be used as a [service worker](https://developer.mozilla.org/en-US/docs/Web/API/Service_Worker_API) for aeppic installations.

## Usage

The default behavior in aeppic is to load a service-worker Layout when rendering `ae-root` which triggers the loading of the service worker from a Preference document.  

## Caching Strategy

The service worker caches all files returned by `/api/v4/statics/_list`.

All files listed in there with `http://` or `https://` are seens as external and cached with [**no-cors** mode](https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API/Using_Fetch).

This actual implementation is here:

::: code-group
<<< @../../../web/service-worker/src/cache.ts#cacheStaticFiles{ts} [web/service-worker/src/cache.ts]
:::
 

All files returned during normal fetches by the clients are also added to the static cache if any of the following conditions are true:

1) They are marked as **immutable** in the Cache-Control reponse header
2) They are cacheable with max-age for at least a day

This actual implementation is here (also checks if response is status ok etc):

::: code-group
<<< @../../../web/service-worker/src/cache.ts#isCacheableResponse{ts} [web/service-worker/src/cache.ts]
:::

 -----
Page Url: src/reference/aeppic-api/api.md
-----
# aeppic HTTP Api
List of all aeppic api endpoints. The api is versioned using the url path and is generated during the build process by fastify

## Version: 3.203.6

### /

#### GET
##### Description

The root of the api

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | Default Response | { **"version"**: string, **"loaded"**: dateTime } |

### /_ping

#### GET
##### Description

Check the server status with a ping request.

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | Default Response | { **"pong"**: dateTime } |

### /auth/access

#### POST
##### Summary

JWT Issuance for Authenticated User

##### Description

Issue a JWT based on provided scopes for an already authenticated user

##### Parameters

| Name | Located in | Description | Required | Schema |
| ---- | ---------- | ----------- | -------- | ------ |
| body | body |  | No | { **"scopes"**: [ string ] } |

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | Default Response | { **"token"**: string } |
| 400 | Invalid request body | string |
| 401 | User not authenticated or invalid scopes | string |
| 500 | JWT issuance failed | string |

### /packages/{packageIdentifier}/_list

#### GET
##### Summary

Returns a plain text of newline seperated file paths inside the package.

##### Description

Lists the files inside a package

##### Parameters

| Name | Located in | Description | Required | Schema |
| ---- | ---------- | ----------- | -------- | ------ |
| packageIdentifier | path | The full **package identifier** based on the `id` of the package document and it's `version` seperated by `@`. Example `945a852b-1573-4783-963a-3f638081c48e@initial` | Yes | string |

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | A list of files inside the package, seperated by newlines (\\n) | string |
| 304 | The package has not changed since the last request. The client can use the cached version of the package. | string |

### /statics/_list

#### GET
##### Summary

Returns a json object with a list of all static files and their hashes.

##### Description

All static files served by the server for the web app can be listed with this endpoint. It can be used by a service worker to precache all static files.

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | A json object with a list of all static files and their hashes. | string |
| 304 | The list has not changed since the last request. The client can use the cached version of the list. | string |

### /features/

#### GET
##### Summary

Returns a json object for all configured features

##### Description

All available and configured features

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | A list of features namespaces and their configuration | object |
| 304 | The list has not changed since the last request. The client can use the cached version of the list. | string |

### /features/namespaces/{namespace}

#### GET
##### Summary

Returns a json object for all configured features

##### Description

All available and configured features in a namespace

##### Parameters

| Name | Located in | Description | Required | Schema |
| ---- | ---------- | ----------- | -------- | ------ |
| namespace | path | The full **namespace** of the features to read | Yes | string |

##### Responses

| Code | Description | Schema |
| ---- | ----------- | ------ |
| 200 | A list of features namespaces and their configuration | object |
| 304 | The list has not changed since the last request. The client can use the cached version of the list. | string |
-----
Page Url: src/reference/aeppic-api/swagger.md
-----
# Swagger

Full [Swagger](https://swagger.io/) YAML version of all routes. The markdown version is at [REST Documentation](./api.md)

<<< @/reference/aeppic-api/api.yaml {yaml:line-numbers}
-----
Page Url: src/reference/aeppic-api/README.md
-----
# API Documentation

The **aeppic** server is fully accessible via a **REST Api**. Everything that aeppic can do is accessible via the API and is used by any service, including the webapp.

Each running instance has a live documentation endpoint while the server is running.

`http[s]://<server>/api/v4/documentation`

::: warning Undocumented Routes
All **v4** routes are documented in here, previous routes are being migrated to v4, but are not listed
here yet. Documentation for those non-versioned routes (v3) are available as a PDF on request.  
:::

In order to interact with the API the user can either use direct HTTP calls (e.g via `curl`) or use the `aeppic` javascript module. That module can be used from Browsers or Node.js scripts and contains all client accessible features of the API. That includes the ability to render the full web application.

::: tip Base URL
All routes described are relative to the base url `/api/v4/`
:::

## REST Routes

All available routes interface is documented in the [REST Documentation](./api.md).

## Swagger

The **@aeppic/api** project uses [Swagger](https://swagger.io/) as part of the build process to create the markdown documentation, but the raw swagger yaml file is also available at [Swagger Raw YAML](./swagger.md).

It can be helpful to access it for processing with other tools.
-----
Page Url: src/reference/aeppic-server/100-data-storage.md
-----
# Install and Update Dataset

Aeppic data is a dataset of json documents and all changes made to it over time. The concepts themeselves are explained in the [Database concepts](../../concepts/100 - Database/README.md) section. Some of the json documents contain links to immutable stored binary streams. These are not stored inside the documents but linked using an URI (`See [Binary data streams](../../../concepts/010%20-%20Data/010%20-%20Binary%20Data%20Streams.md))`.

-----
Page Url: src/reference/aeppic-server/020-web-server.md
-----
# Web Server

## Injected Headers

All requests to the HTTP/S endpoint return the following HTTP Headers:


|Name                   |Example Value  |Description          |
|-----------------------|---------------|---------------------|
|X-Aeppic-Environment   |development    |This is the NODE_ENV variable at start, which influences some settings. Especially the served files|
|X-Aeppic-Version       |aeppic/3.92.4 @aeppic/server/3.123.5 express/4.17.1||

-----
Page Url: src/reference/aeppic-server/001-install.md
-----
# Installing

An aeppic server consists of two parts. The actual services required and the dataset within that instance. 

The primary service required is the `aeppic-server` service which can be configured and exposes the webserver apis.

## Installing service via npx

Given a node.js runtime environment (`^16.13.0`)

`npx @aeppic/install-server [--token=<ACCESS_TOKEN>] [--data=<DATA_PATH>] [<TAG|SEMVER>] `

`--token` is the unique access token required to access the repository server
`--version` declares the version to install (can only be a newer version)

`--data` can be left off during subsequent updates but specifies the absolute path to the primary data storage folder

`[<TAG|SEMVER>]` is an optional parameter. It defaults to **stable**

> Declaring parameters as environment variables also works
> e.g. `AEPPIC_INSTALL_VERSION`, `AEPPIC_INSTALL_DATA`

<!-- ## Rollback

In case of a mistake or issue with a newly installed update run `npx @aeppic/install-server --rollback` which rolls back the last installed version. Use `--rollback-data` to also undo any data changes made since installation. Since this will actually delete data, a prompt will appear. Override using `--force`. -->

### Installing service from a private repository server

The binaries for the installation are retrieved from a repository server.
By default that server is located at `https://get.aeppic.com`, but can be overridden via the `--server` argument.

`npx @aeppic/install-server --server=<REPOSITORY_SERVER_URL>  [--data=<DATA_PATH>] [<SEMVER|tag>]`

Note: Declaring `AEPPIC_INSTALL_SERVER` as environment variable also works

### Install and run a private repository server

A repository server is a simple static file webserver. It uses Bearer Authentication to authenticate downloads and uploads.

`npx @aeppic/install-repository-server [--data=<LOCAL_DATA_PATH>] [--port=<PORT>] [--install-location=<LOCAL_INSTALLATION_LOCATION>] [--upload-token=<UPLOAD_TOKEN_JSON_PATH>] [--download-token=<DOWNLOAD_TOKEN_JSON_PATH>]`

> Run via docker

Once config and data is installed and with access to the container registry it can be started via: 

`docker run -it --rm -p 8088:8088 --mount type=bind,source="/etc/opt/aeppic",target=/etc/opt/aeppic --mount type=bind,source="/opt/aeppic/data",target=/opt/aeppic/data --platform=linux/arm64/v8 registry.dev.curasystems.com/aeppic/aeppic-full/server:latest`

To install data:

`AEPPIC_DOWNLOAD_TOKEN=<TOKEN> node bin/aeppic-server.js install  --data-only --data-version=latest`-----
Page Url: src/reference/aeppic-server/025-web-server-api.md
-----
# API

aeppic's HTTP Api has two entry points `/api` and `/api/v4` where the former is the current main entry point until all calls have migrated to the
later versioned api. 

## -----
Page Url: src/reference/aeppic-server/005-server-configuration.md
-----
# Configuration
 
Almost all configuration is done inside of Aeppic documents itself (Which services to run etc). Some settings though that are web server or node specific, are contained in a configuration file. These pertain mostly towards which port to listen to, which SSL certificate to use if any, upload bandwidth limits, overload protections, info as to the returned index.html etc.

The environment the server starts with, is also influenced by systemd usually located at [/etc/systemd/system/aeppic-server.service](#systemd-default-configuration)

::: warning Security Sensitive
Some settings can have serious consequences when not correctly configured !

Do not change settings like [Custom Headers](#custom-headers), the [Trusted Proxy](#trusted-proxy), or others without understanding the implications.
:::

## Location

Default location is `/etc/opt/aeppic/server.json` in JSON format.

Fallback defaults can be found in the [`server.json`](#full-default-configuration)
located in the main server directory.

## HTTP/S Server

These keys configure the HTTP and HTTPS server parts.

The `hostname` is used to know which fqdn the server is accessed by. In case of using a self-signed certificate its also the common name.
When `port` is not null it will activate the http server on that port and on the specified `ip`. If hostname is not defined the machine
hostname is used.

The `https` settings in the `https` section are identical, but https needs to be enabled with `https.enabled` set to `true`. `key` and `cert` have
to point to resolvable pem files. If the paths don't exist a self-signed certificate is used. If the files are not readable the server will raise
an exception.

If the server is to be dually hosted under http and https without just redirecting the http traffic to https, the key `allowHttpNoRedirect` can be set to `true`.
This scenario can be helpful if the server is in an intranet and uses https while also being visible from behind a terminating proxy. Or if https is used from the server directly, but the API users such as servcies can work without SSL.

> Running aeppic-server just with http is only sensible for some dev scenarios on localhost usually as a lot of web features require https or localhost.

```
  "hostname": "",
  "port": 80,
  "ip": "0.0.0.0",
  "https": {
    "enabled": false,
    "ip": null,
    "port": 443,
    "key": null,
    "cert": null,
    "allowHttpNoRedirect": false,
  },
```

## Api Url

In certain scenarios, such as intranet settings, it is helpful to have a way to define which API Url for internal services to use instead of using the public hostname. 

```
  "api": {
    "url": "http://localhost/api"
  }
```

## Data Location

```
  "dataDir": "/opt/aeppic/data"
```

### Source Data

It is possible to define a second data directory as a read-only fallback to source data from.

```
  "sourceDataDir": "/opt/aeppic/stage/my-test"
```

This is used to specify an alternate data store which base data and changes are read from and combining them with data from the main dataDir. 

- First it tries to load the base file from the `${dataDir}/base` folder.
- If no system file is present it will try to load from `${sourceDataDir}/base` folder.
- Then it will load all changes from  `${sourceDataDir}/changes`
- Then it will load the standard changes from `${dataDir}/changes`

This feature can be used to start a new version of the server with existing data without altering it. It is used by the [Staging feature](/architecture/Other%20Features/README#staging)

**Source Policies**

When loading from the specified source data a policy can be applied to limit loading 

```
  "sourceDataPolicy": "latest",
  "sourceDataPolicies": {
    "latest": {...},
    "myPolicy": {...}
  }
```


## Features (Feature Flags) {#features}

Feature Flags play a pivotal role in managing the evolution of software (See [article](https://de.wikipedia.org/wiki/Feature_Toggle)). They are designed for use in continuous delivery pipelines, allowing new features to be introduced and deprecated progressively. This approach provides flexibility, enabling developers to test new features in a controlled environment before a full-scale rollout.

It's important to distinguish Feature Flags from [Preferences](../../concepts//050%20-%20Preferences/README.md). While they might seem similar, they serve different purposes. Preferences are settings that can be adjusted globally or on a per-user basis to alter the default behavior of certain features. They provide users with the ability to customize their experiences based on their needs or preferences.

For instance, let's imagine we're adding a new "Dark Mode" feature to our application. Using a Feature Flag, we can control the release of this feature, enabling it for a small subset of users or certain environments first for testing and gradual rollout.

The Preference, on the other hand, is what allows users to turn on the "Dark Mode" once it's rolled out. Some users might prefer the traditional "Light Mode", so they don't change this setting. Others, especially those who often work at night or in low-light conditions, might prefer the "Dark Mode" and choose to enable it in their user settings.

In this way, Feature Flags control the deployment and access to the feature, while Preferences allow users to customize their interaction with it.

**Naming rules**

When defining a feature namespace or name, certain characters, such as : or @, are not permitted.

The aeppic client uses the aeppic namespace to manage its [Features](../../reference/aeppic/modules/features.md) module. If you're developing custom applications on aeppic, you can extend this with namespaces that are specific to your application. Please note that the aeppic server reserves the namespaces beginning with @aeppic/*, which are used to control specific server features.

To avoid namespace conflicts, it's crucial to ensure your namespace is unique. We recommend prefixing it with your registered npm organization name or web domain. This approach not only ensures uniqueness but also helps in identifying the source of the feature.

<!-- As a convencience there is an override segment in which a feature can be read disabled or enabled for
a specific account. This is usually NOT part of the main config file, but used in conjuction with the
**config-override** option.
<--
"override": {
      "accounts": {
        "some-account-id": {
          "some-namespace:feature-name": true, // To enable a feature
        }
      }
    },-->

```json
{
  ...
  "features": {
    "aeppic": {
      "feature-name": {
        "enabled": boolean // Optional. Defaults to `true`
        "description": string // Optional
        "options": object // Optional
        "deprecated": boolean // Optional
        "deprecationNotice": string // Optional
      }
    },
    "@aeppic/api": {

    },
    "<namespace>": {
      ...
    },
  }
  ...
```

## Static Files

The Web Server has to serve a number of static files across multiple locations.
This is configured in server.json under `statics`. It has three parts:

1. always
2. development
3. production

```json
{
  ...
  "statics": {
    "always": [...],
    "development": [...],
    "production": [...]
  }
}
```

`always` is always applied, `development` is served when `NODE_ENV` is set to `development` and `production` is the alternate used when `NODE_ENV` does not match development.

Each section defines urls matched to files located relative to the aeppic server directory. 

Either it exposes a complete folder under an url prefix:

```json
{ "prefix": "/aeppic/libs", "folder": "node_modules/aeppic/libs" }
```

or it exposes a single file with a specific path

```json
{ "url": "/main.js", "file": "dist/dev/main.js" }
```

Some files are not relevant for service worker caching (e.g `.js.map` files). Those files or folders can be exempted with `"offline": true`. They still need to be listed to be available on demand via normal static content serving.

```json
{ "url": "/main.js.map", "file": "dist/dev/main.js.map", "offline": false }
```

### Advanced options

The option `refreshStaticList` can be set to `every-call` which causes the `statics/_list` API call to always run
a full filesystem enumeration on every call. This is a rather expensive operation and should only be done during development.

Not having this option set to `every-call` means the server will not realize when assets change in the filesystem after startup in the `statics/_list` call, but changed files and new content will still be served correctly as it only affects the list. Since the static files usually do not change after boot this option is only really helpful for development. 
 
```json
{
  "refreshStaticList": "every-call",
  "always": ...
}
```

<!-- In the case it exposes an entire directory, if it does not find a match it will fallthrough to the next rule by default. This can be overriden via `fallthrough` set to `true`.

```json
{ "prefix": "/", "folder": "webroot", "fallthrough": true }
```
This is important for the `/` path as it is the last catch all rule before serving the [SPA page](#spa).
-->

::: info Note
This section is very important for the offline client behavior. All static path rules are exposed via the API for a service worker to fill its offline static file cache. 
:::

### Caching

The caching rules by default use [ETag](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/ETag) behavior for all files with no other form of caching allowed. Improved caching should be handled by a service worker to ensure consistent assets.

## Single-Page Application (SPA) {#spa}

The template to server when falling through all static paths is configured in the `html` section.

```json
{
  ...
  "html": {
    "index": "./server/index.template.html"
  }
}
```

It is possible to further fine-tune the template instead of just replacing it fully. For that you can
override stylesheet sets and their import order and scripts to be loaded.

```json
{
  ...
  "head": {
    "stylesheets": {
        "sets": {
          "icons": [
            "/libs/fontawesome-pro-5.15.4-web/css/svg-with-js.min.css"
          ],
          "legacy": [
            "/aeppic/content/aeppic.min.css"
          ],
          "next": [
            "/dist/main.css"
          ],
          "loading": [
            { "file": "/dist/loading.css", "inline": true }
          ]
        },
        "load": ["legacy", "icons", "loading"]
        
      },
      "scripts": {
        "load": [
          "/aeppic/libs/less/dist/less.min.js",
        ]
      }
    },
  },
  ...
}
```

## Trusted Proxy {#trusted-proxy}

The trusted proxy is required to ensure X-Forwarded-For headers are trusted and
used for origin IP resolution in logs and for certain security decisions from outside
sources. 

::: warning 
The IP address of the accessing client is used not just in logging but also when making decisions requiring trust. You have to ensure the proxy is actually trusted.
:::

> A simple ip

```json
{
  "trustedProxy": "10.0.0.1"
}
```

> An array of simple ips or net masks

```json
{
  "trustedProxy": ["10.0.0.0/8"]
}
```

## Trusted Admin Ips

Certain administrative calls are only allowed from trusted ips (after taking into account the [Trusted Proxy](#trusted-proxy) settings). 

> Can be a simple IP (IPv6 is supported) or net masks (see [Trusted Proxy](#trusted-proxy))

```json
{
  "trustedAdminIps": ["127.0.0.1", "::1" ,"::ffff:127.0.0.1"]
}
```

## Upload Limits

In order to limit uploads speeds and to avoid server overloading, an upload throttle
can be configured. It will throttle all incoming streams to a total maximum desired
throughput rate. This maximum rate applies to all uploads at the same time which are
not specifically exempted. So 100 MBit/s could mean 5 * 20 MBit/s.

Uploads made by certain users, ip-ranges, admins can be exempted and are configured
by default. External services can thus continue uploading at maximum available speed.

The throttle is configured in Megabits per second (MBs with base 10, NOT MiB/s base 2).

```json
{
  "bandwidth": {
    "uploadThrottleInMegabitsPerSecond": 100,
    "exemptAdminsFromThrottle": true,
    "exemptIpsFromThrottle": ["10.1.0.0/16", "127.0.0.1", "::1" ,"::ffff:127.0.0.1"],
    "exemptAccountsFromThrottle": ["some-account"]
  }
}
```

## Too Busy

When a server is very busy, certain measures might be necessary to limit access to the system.

The configuration section `new_session_limits` manages the behavior of accessing the main 
web application. It is triggered whenever the website/app is opened in a new browser tab. 
So any web request to the website itself where the URL route is not handled by a
static file response or the api.

The child `concurrent_uploads` when enabled allows monitoring the number of concurrent uploads
and cause the new session limit to be activated.

It triggers whenever more files are uploaded than configured and:

  1) The call is not originating from an exempted ip range
  2) The caller is not already authenticated by a previous visit
  3) The caller is authenticated but does not match the configured exceptions 

then the configured html page is returned.

The default `busy.html` tries a reconnect after a few seconds and triggers a
full reload once the limit has been lifted.

> Note that `maxUploads` is independent of bandwidth actually used.

```json
{
  "toobusy": {
    "new_session_limits": {
      "concurrent_uploads": {
        "enabled": true,
        "page": "/busy.html",
        "options": {
          "maxUploads": 20,
          "doNotLimitAdmins": true,
          "doNotLimitAccounts": [],
          "doNotLimitIps": ["10.1.0.0/16", "127.0.0.1"]
        }
      }
    }
  }
}
```

## Descendant TimeStamp tracking

By default the server tracks whenever a document is changed (which is reflected in the document.t field).
In many cases it would be helpful to know the latest change of a descendant document in the tree.
E.g. when is the last time a descendant of e.g. an app was changed.

This is not without some overhead if we did this for every document and updated the time in every
ancestor. Therefore we only track this if the document is a descendant of a certain form.

The tracking itself is done via in memory tracking and not automatically injected into the document.
The information can be read via the query API by setting `dts: true` as part of the query options.

**Configuration**

```json
{
    "features": {
        "@aeppic/server": {
            "descendantTimestamps": {
                "enabled": true,
                "description": "Track descendant timestamps",
                "options": {
                    "fullTrackWhenDescendantOf": [ "system", "apps" ],
                    "fullTrackWhenDescendantOfForm": [ "application-form", "form-folder" ],
                    "minTrackDepth": 2
                }
            },
        }
    }
}
```

## Threaded InMemory API 

By default the in memory api runs in a separate worker thread, this can be configured or disabled via

**Configuration**

```json
{
  "features": {
    "@aeppic/server": {
        "inMemoryApiOnThread": {
            "enabled": true,
            "options": {
                "allowAttachDebugger": true,
                "enabledDebuggerSignal": "SIGUSR2"
            }
        },
        ...
    },
    ...
  }
}
```

By default that also allows to attach a debugger onto this thread when SIGUSR2 is received by the main process.

In addition when using a threaded api calls to the export function (for example used by upload garbage collecting ) it is important to not do this synchronously on the model thread as that would block the thread for running queries.
If the export was run via the api proxy we thus inject pauses into the enumeration which is configured via the following settings.

```json
{
  ...
  "apiThread": {
    "iteration": {
      "blockSize": 50,
      "blockDelay": 10
    }
  },
  ...
}
```

This will configure the thread proxy to enumerate in blocks of **50** (sent via the thread worker bus to the calling thread) and then delay by **10ms**.

## Startup

In some circumstances it is convenient to start a machine in a test environment but to ignore
some recent changes. The syntax that can be used is the one supported by [parse-duration][npm_pd]

> Beware:  Use **ONLY** in test environments with a full backup available !
>
> Changes at the cutoff date and later could be overwritten, so going
> backwards in little steps is advised.

```json
{
  "startup": {
    "ignoreRecentChanges": "-30d"
  }
}
```

[npm_pd]: https://www.npmjs.com/package/parse-duration

## Ignore system/app changes

After loading the base files from the base/active path changes
are being applied on top of that. With `ignoreAppSystemChanges` it
is possible to ignore any changes that try to modify changes located
in the **/apps** or **/system** folders. 

This is especially helpful when transporting new apps or system versions
to an existing machine where no changes were supposed to have happened
anyway.

> Beware that any changes that should be kept would have to be 
> transported backwards.
>
> Also any system configured like this, will reset any system or apps
> changes with every reboot

```json
{
  "startup": {
    "ignoreAppSystemChanges": false
  }
}
```

## File Reference statistics

With `trackFileReferenceStats` the in memory model can be configured to track file usage statistics in the `fileReferences.stats` file located in the data directory.

::: details
When talking about files this refers to file fields inside documents.

```md
[My File][file]

[file]: File
```
:::

```json
{
  "stats": {
    "trackFileReferenceStats": true
  }
}
```

The statistics can be used by `aeppic-files` (located in the servers `bin` folder) to create
information about which uploaded binary files are still in use or which files referenced files
are missing.

The statistics file is a SQLite3 database and is defined by `file-reference-stats.ts`

## Custom Headers {#custom-headers}

::: warning
Custom headers can override certain security sensitive behavior in browsers. Any changes here should only be made after thinking through the implications.

It is used for example to allow serving service worker scripts from dynamic file field which require a certain HTTP header.
:::

### Allow custom headers for _content

Sometimes there is a need to return specific HTTP headers when reading data from the _content endpoint reading dynamic data. The `headers` section allows to configure this.

#### Example to allow Service Worker File Content

In case a service worker script is desired to be loaded from dynamic content (e.g using the `Aeppic.Content.getSrc` ) the script would not be able to be registered at a higher scope [(See here)](https://developer.mozilla.org/en-US/docs/Web/API/Service_Worker_API/Using_Service_Workers) without being hosted at an appropriate root level by the web server or including the `Service-Worker-Allowed` header. That header is not a standard header, but is defacto required and supported by all modern browsers.

In order to allow dynamic loading of service workers there is a `includeServiceWorkerAllowed` setting.

E.g with the configuration below With it you can configure a matching rule to allow including it with requested content. When a `/.../_content` GET request comes in which includes the query parameter `headers=service-worker` then the two headers are added **if** the document is a descendant of the ancestors defined in `ancestorIsAnyOf`.

This header setting 

```json
{
  "headers": {
    "service-worker": {
      "include": [
        [ "Service-Worker-Allowed", "/" ],
        [ "Content-Type", "text/javascript" ]
      ],
      "require": {
        "ancestorIsAnyOf": ["system", "bd95c380-480e-41f9-88dd-790f143ab0f8"]
      }
    }
  }
}
```

### Dynamic component Lookups

Designs and controls are dynamic components which are looked up by name. The lookup can be configured to use a different strategy or to cache the results.

The full strategy is explained in [Design](../../concepts/020%20-%20Rendering/020-designs.md#lookup-rules) and [Controls](../../concepts/020%20-%20Rendering/015-controls.md#lookup-rules)

| Option | Description |
|--------|-------------|
| `strategy` | The strategy to use for lookup. Can be `full`, `system`. Default is `classic` for now |
| `cache` | Whether to cache the results. Default is `true` |
| `fallback` | Whether to use the previous strategy. Default is `false` |
 
```json
  "control-lookup": {
      "enabled": true,
      "description": "Lookup controls with an improved search algorithm.",
      "options": {
        "strategy": "classic",
        "cache": true,
        "fallback": false
      }
    }
```

> Note: If the feature is turned on and set to classic strategy,
> it will use the new and classic strategy in parallel and compare the results before returning the classic result. Discre


| Option | Description |
|--------|-------------|
| `strategy` | The strategy to use for lookup. Can be `full` or `classic`. Default is `classic` for now |
| `cache` | Whether to cache the results. Default is `true` |


```json
  "design-lookup": {
    "enabled": true,
    "description": "Lookup designs with an improved search algorithm.",
    "options": {
      "strategy": "classic",
      "cache": true
    }
  }
```

## SystemD default configuration

<<< @/../../server/server/aeppic-server.service {ini:line-numbers}

## Full Default configuration

<<< @/../../server/server/server.json {json:line-numbers}
-----
Page Url: src/reference/aeppic-server/README.md
-----
# Overview

The aeppic server consists of the server processes and the dataset it operates on.-----
Page Url: src/reference/aeppic/Forms/010 - form.md
-----
# Overview

[[toc]]

## Parsing formdown

Forms are defined via [PEG.js](https://pegjs.org/) syntax.

::: details parser-syntax.pegjs

::: code-group 
<<< @../../../libraries/forms-parser/src/parser-syntax.pegjs {js}
:::

> Note: The formdown parsing rules are provided by the `@aeppic/forms-parser` module. 

## `class Form`

Aeppic is based on Forms. The `Form` class handles parsing the form document definition, building the form controller
and form functions. 

It uses the parsed form info to expose a structured view of the form via it's `rootSection` property. Which returns a `FormSection` (Related to, but not identical to the raw `ParsedFormSection`).

The class is exposed whenever working with [EditableDocuments](../editable-document.md) via the `form` property:

```ts
const myDocument = await Aeppic.edit('....')
myDocument.form.name
```


### Using Tags on fields {#tags}

When handling e.g optional fields they 

Each tag where interpretations might interfere (e.g shared forms etc) should be namespaced with a unique preceding prefix followed by a colon. E.g `[myField]: <my-company:my-tag>`.

::: warning Reserved tags
The following tags are reserved by aeppic: 

* **required**
* **optional**
* **sensitive**
* **encrypt**
:::

> Note: The `EditableDocument` exposes the same functions by forwarding the call to it's form.

It is possible to query by name or value with string or regexp.

#### isTaggedField

::: code-group
```js [code]
const isTagged = form.isTaggedField('myField', ['tag1', 'tag2'], { operation: 'or' })
// isTagged === true

const isTagged2 = form.isTaggedField('myField', ['tag1', { name: 'tag2' }], { operation: 'and' })
// isTagged2 === false

const isTagged3 = form.isTaggedField('myField', ['tag-x', { name: /^tag/ }], { operation: 'or' })
// isTagged3 === true
```

```md [form.md]
[My Field][myField]

[myField]: <tag1>
```
:::


#### `getTaggedFields`

::: code-group
```js [code]
const taggedFields2 = form.getTaggedFields([{ name: 'tag1', value: /^a/ }, 'unused-tag'], { operation: 'or' })
// taggedFields2 is ['myField2']
```

```md [form.md]
[My Field][myField]

[myField]: number <tag1=a-value>
```
:::

#### `getTaggedSections`

This function recursively goes through all sections from root down and finds sections where all fields match the requested requirements.

Sections with fields that directly have placeholders to fields which match are returned (or rather their indices are).
Sections with no fields at all wont be included (they don't match)
Sections with with subsections only get returned when all their subsections are matching the criteria above. Independent of whether 
it itself has fields that match or not.

::: code-group
```js [code]
const taggedSections = form.getTaggedSections(editable, ['optional'])
// taggedSections is [1]
```

```md [form.md]
[My Field][myField]

# A Section
[My field 2][myField2]

[myField2]: <optional>
```
:::


### Handling undefined fields

Undefined fields are allowed when the form is marked to allowUndefinedFields.

A field is defined as undefined when the field's value is equal to undefined. The functions

`form.getUndefinedFields(document)`, `form.getUndefinedSections(document)`, `form.isUndefinedField(document, fieldName)` thus both take a
document as input. They are also exposed by the editable document itself (without the parameter of course).

Otherwise they behave identical to the function to search for tagged fields [above](#tags), with the difference being
the match criteria of a placeholder/field being the value of the field being `undefined`.  

> Note: The `EditableDocument` exposes the same functions, but without the document parameter using itself instead.

## Types

::: details
<<< @../../../libraries/forms-parser/types/index.d.ts
:::

## Examples of Parsed Info   

```js
info = {
   conditions: [...],
   default: { ... },
   definition: "...<formdown>...",
   fields: [...],
   log: (...),
   placeholders: (...),
   schema: (...),
   sections: [],
}
```

#### Field

```json
{
   cardinality: (...),
   defaultValue: (...),
   description: (...),
   filter: (...),
   name: (...),
   required: (...),
   settings: (...),
   subType: (...),
   tags: (...),
   type: (...),
}
```

#### Section

```js
rootSection = {
   "offset": 0,
   "offsetExcludeDirectives": 0,
   "offsetExcludeTitle": 0,
   "length": 0,
   "lengthIncludeSubsections": 9054,
   "level": 0,
   "title": "",
   "type": "ROOT",
   "style": {},
   "placeholders": [],
   "paragraphs": [],
   "index": 0,
   "directSubsections": [subSection]
}
```

```js
subSection = {
   "offset": 0,
   "offsetExcludeDirectives": 0,
   "offsetExcludeTitle": 13,
   "length": 2070,
   "lengthIncludeSubsections": 3958,
   "level": 1,
   "title": "Allgemein",
   "type": "ATX",
   "style": {},
   "placeholders": [...],
}
```

#### Paragraph

```js
paragraph = {
   "offset": 0,
   "length": 19,
   "hasText": true,
   "hasPlaceholders": true,
   "placeholders": [
      {
            "field": {
               "name": "field",
               "cardinality": null,
               "defaultValue": "",
               "description": null,
               "filter": null,
               "type": "string",
               "subType": "text",
               "tags": {},
               "settings": {
                  "help": {
                        "text": "",
                        "html": ""
                  }
               }
            },
            "offset": 5,
            "length": 14,
            "label": "field",
            "control": null,
            "style": {},
            "index": 0
      }
   ],
   "style": {},
   "offsetExcludeDirectives": 0
}
```

#### Placeholder

```js
placeholder =  {
   "field": (...),
   "offset": 13,
   "length": 36,
   "label": "Bezeichnung der Verarbeitung",
   "control": null,
   "style": {},
   "index": 0
}
```

#### Control

```js
placeholderControl = {
   "namespace": "",
   "name": "ref",
   "fullName": "ref",
   "parameters": {
      "hide-path": {
         "value": "true"
      },
      "ancestor-search-limit": {
         "value": "2"
      },
      "handle-selected": {
         "value": "false"
      }
   }
}
```

#### Field

```js
field = {
   "name": "zulaessig",
   "cardinality": {
         "max": "n",
         "min": "0"
   },
   "defaultValue": {
         "text": "",
         "id": "",
         "v": ""
   },
   "description": null,
   "filter": [
         {
            "f.id": "'e86d03d7-1711-4138-b4ee-990efbd1a988'"
         }
   ],
   "required": null,
   "type": "object",
   "subType": "ref",
   "tags": {},
   "formRef": "e86d03d7-1711-4138-b4ee-990efbd1a988",
   "settings": {
         "help": {
            "text": "",
            "html": ""
         }
   },
   "joinedFilter": "f.id:'e86d03d7-1711-4138-b4ee-990efbd1a988'"
}
```-----
Page Url: src/reference/aeppic/writer.md
-----
-----
Page Url: src/reference/aeppic/document.md
-----
# Document

Documents in aeppic are represented in aeppic as a JSON data structure:

The document type is specified in `@aeppic/types`.

::: details Document
<<< @/../../libraries/types/index.d.ts#document{ts:line-numbers}
:::

The document's actual user data is stored in its `data` field.

::: details Data
<<< @/../../libraries/types/index.d.ts#data{ts:line-numbers}
:::

Which fields are actually stored in the document is defined by the form the document is based on.
The form is used as the type or schema of the document's data.

> Forms are also documents and explained in more detail [here](../../concepts/010%20-%20Documents/005-forms.md)

## Metadata

Most important metadata is stored directly inside the document object.

E.g a reference to the form of the document is stored in its `f` field for example.

```json
{
  ...
  f: {
    id: 'FORM-ID',
    v: 'FORM-VERSION',
    text: 'FORM-NAME'
  },
  ...
}
```

Or the last time the data of the document was modified is stored in its 'modified` field.

```json
{
  ...
  modified: {
    at: '',
    ...
  },
  ...
}
```


But some more seldomly used metadata is stored in the `meta` field. For example


::: details Metadata Type
<<< @/../../libraries/types/index.d.ts#meta{ts:line-numbers}
:::

-----
Page Url: src/reference/aeppic/editable-document.md
-----
# Editing Documents

## Create a new document

```javascript
const d = await Aeppic.new('document-form', 'parent-document')
d.data.name = 'New document'
Aeppic.save(d)
```

## Edit an existing document

```javascript
const d = await Aeppic.edit('document id')
```

## Insert or update a document

Sometimes it is helpful to create a document exactly once and just update it with the latest
available data. Create it when needed or update it should it already have existed. 

In cases like this the document needs a predictable identifier. It is best to derive it from
the documents parent id with a well-known string. The function
[`Aeppic.Functions.deriveChildId`](./modules/functions.md#derivechildid) generates that
by concatenating the arguments in a standardized fashion for that purpose. 

```javascript
const d = await Aeppic.new('document-form', 'parent-document', {
  id: Aeppic.Functions.deriveChildId('parent-document', 'my-service', 'abc')
  insertOrUpdate: true
})
d.data.name = 'My Name'
Aeppic.save(d)
```



-----
Page Url: src/reference/aeppic/aeppic.md
-----
# Aeppic Interface

The `Aeppic` object is the central point to interact with aeppic via code.

It has a couple of different interface variants depending on the context it is used in.

| Name | Description |
|---|---|
|`AeppicInterface`| The most basic interface the others build on |
|`AeppicUiInterface`|The interface available to UI components which has some extra UI related features|

The `AeppicActionInterface` limits the writes to Aeppic. They are not performed directly but put into a holding block. When the action finishes without error and does not time out (e.g sync timeout or promises take too long) the writes are then sent to Aeppic.

Most function in Aeppic Interface are async. Where possible they do make use of caching and might be resolved i.e synchronously, but that is just a performance optimization. Always use `await` to interact with these methods.

A notable exception is `save` or `saveAll`. Those calls will be handled as soon as possible, but you cannot wait directly for the them to have an effect. To handle side-effects of changes Aeppic uses `BusinessRules` (If-Then Rules).

Thus `save` and `saveAll` do not return anything. In case the changes have some syntactical error or the system cannot even store the changes for transmission etc an exception will be raised, but most users will never have to handle that. Reason being, that `save`/`saveAll` is often called from within Commands, Business Rules, or UI components and these already handle exceptions of this type. 

The functions of an Aeppic instance are logically structured into a few sections.

Some are just a collections of functions (like those handling Documents) and some
(like Commands/Features etc.) are exposed as Module properties e.g `Aeppic.Features`. 

## Documents

> These functions deal with documents in the tree. Creating, Reading, Updating, Deleting (CRUD)
> 
> Documents are identifier either by its id (the `document.id` field) or by using a
> reference to the document. 

> A reference is an object which contains an id `{ id: '<id of document>', v: 'version of document>' }` 
> and often also contains a version property (`v`) and a text property that describes the document
> referenced (usually generated from the name of the document at the time the reference was generated).
> The version is ignored in many cases and the newest version of the document is returned, but in
> cases of forms or inside the history of a document it often becomes quite relevant. 


#### `async new(formIdOrReference, parentIdOrReference, options?)`

This will create a new document (but not save it yet) based on the designated form and located underneath the designated parent.

It returns an `EditableDocument` to then fill the document with the appropriate data.

#### `save(editableDocument)` / `saveAll(editableDocuments[])`

All these functions that do potentially write to the aeppic document model do NOT
return a success code. All changes are queued up and flushed internally at a
later point in time. 

> In the case of UI component controllers the flush happens at the end of each exposed function, 
> in the case of e.g business rules or other actions it happens once the code ran through.
> 
> The advantage for this is that logical errors during the excecution of a handler will not even 
> attempt to change aeppic and all changes are transmitted atomically to the server.
>
> Once the server has received the changes they will be handled eventually, though these might
> not result in actual changes to any document. 

#### `saveDetached` <Badge type="warn" text="Internal" />

> An internal function. Do not use

#### `async get(documentIdentifier)`

Get the most recent version of the document. The returned document is read-only. Returns null
when a document does not exist (or is not accessible by the current user).

#### `async getAll(documentIdentifier[])`

Get the most recent version of all defined documents. When a document does not exist etc null is
returned in the resulting array.

#### `async getForm(formIdentifier)`

Get the exact form specified or the latest if the identifier is just the id string. 

#### `async getFormForDocument(documentIdentifier)`

Get the exact form used by the document. 

#### `async edit(documentIdentifier)`

Get an editable version of the document. In order to edit a document the form of the document
is also loaded internally in order to execute ensure data correctness and executing all rules.

#### `move(documentIdentifier, targetIdentifier)`

Move is a function which does not return  a value (similar to save). Aeppic will
do its best to safely send the command to the server. 

#### `delete(documentIdentifier)`

Moves a document into the `recycler` document.

#### `deleteHard(documentIdentifier)`

Deletes a document directly, without moving it into a recycler. Such a deleted
document is not easily restorable.

#### `change(documentIdentifier, targetFormIdentifier)`

Change the form of a document to the targeted form. When a version is defined in the target
form identifier (if it is a reference) then the exact form will be used.

#### `async canUpgrade(documentIdentifier)`

When the form of a document has a newer version this returns true

#### `upgrade`

When the form of a document has a newer version `change` is called with that newer version.

## Searching 

#### `find`
#### `findOne`
#### `query`
#### `hasChildren`

### `Query`

In order to build queries, not with strings that need to be parsed,
the `Query` property allows access to fluent builders for queries.

These are preconfigured where possible, eg. called in a UI context
the `Query.children()` builder is already rooted with the document
currently being represented. 

#### `Query.children()`
#### `Query.descendants()`A
#### `Query.global()`

## Watching / Subscribing

#### `watch(documentIdentifier)`
#### `subscribe(queryString)`
#### `updateSubscribtion`

## Stamping

#### `stamp(target, stamp, options?)`

#### `verifyStamp(target, stamp)`

## Auth

#### `login()`

Allows to change the authentication of the server

#### `logout()`

Allow to clear all authentication data and change to previous login. A UI usually goes back
to the login screen, but in some cases it could also revert to a group login.

#### `Auth`

#### `Account`

#### `GroupAccount`

#### `hasRight`

#### `HomePage`

#### `ProfilePage`

#### `getProfilePageForAccount`

#### `getAccountForProfilePage`

## Navigation <Badge type="info" text="AeppicUiInterface" />

#### `navigateTo(documentIdentifier)`

Changes the current UI route to a document .

#### `drillInto(documentIdentifier)`

This might change the route, but can be intercepted by e.g Dialogs to display the new document
inside the dialog.

#### `navigateBack()`

## History <Badge type="info" text="AeppicUiInterface" />

 
## Content

## Commands

## Actions

## Logging

## Packages

## Preferences

## Utility

#### `sort(list, cb)`
#### `uuid()`
#### `download()`
#### `asReference()`
#### `newDOMId()`

#### `setInterval()`
#### `clearInterval()`
#### `debounce()`
#### `copyToClipboard`

## Features <Badge type="warn" text="Unstable" />

At the moment this is an internal module not to be used outside of
aeppic. It handles feature flags for use within Aeppic.

## File Handling <Badge type="info" text="Desktop Client Only" />

```typescript
async Aeppic.editFile(
  url:string,
  fileName: string,
  documentInformation: {
    id: string,
    v: string,
    fieldName: string,
    mimeType: string,
  })
```

The first time this is called the Aeppic instance also subscribes
to edit events.

### Events raised

You can subscribe to events emitted during editing via

`Aeppic.addEventListener('client:files:edited', event => {...})`

|Event|Type|Description|
|-|-|-|
|files:changed|FileChangeEvent||
|upload:started|FileUploadEvent||
|upload:canceling|FileUploadEvent||
|upload:canceled|FileUploadEvent||
|upload:failed|FileUploadEvent||
|upload:finished|FileUploadEvent||

Event data:

```typescript
{
  type: string,
  document: { id: string, v: string, text?: string },
  fieldName: string,
  localFilePath: string,
  ...
}
```

::: details FileEditEvent (aeppic.ts)
<<< @../../../libraries/aeppic/src/aeppic/aeppic.ts#FileEditEvent
:::

## Developer <Badge type="warn" text="Internal" />

## Web <Badge type="info" text="AeppicUiInterface" />

#### `renderLayout(layoutIdentifier, options)`

#### `fetch`

A preconfigured version of fetch. Which ensures the call
is properly authentiated and uses the version of fetch
passed into Aeppic during construction when defined.

#### `async nextRenderPass(callback)`

In some UI contexts it is helpful to wait for a render pass
to ensure we can continue execution. This call waits for the UI
renderer to go through one render cycle. In Vue this would
be `nextTick`.

#### `Vue`

An accessor to the Vue instance used by Aeppic. This is ONLY to be
used for advanced usage scenarios.

#### `triggerResize`
#### `dispatchDOMEvent`

## Storage <Badge type="info" text="AeppicUiInterface" />

#### `LocalStorage`
#### `SessionStorage`

Both have a `isAvailable` property to check whether they are even available.

## Translation

## Date

DateTime is a version of `Luxon` which helps with formatting dates etc. 

#### `DateTime`

#### `DateTimeDuration`

#### `DateTimeInterval`

#### `DateTimeInfo`

#### `setDateTimeLocale(locale)`

## Parser

#### `Parser.markdown`

#### `Parser.formdown`

#### `Parser.dateRange`

## Modules (Extended Modules)

#### `Modules.FileSaver`

#### `Modules.Popup`

## Functions 

#### `Functions.resolveField`

#### `Functions.sort`

#### `Functions.deriveChildId`

## Routing  <Badge type="warn" text="AeppicUiInterface" />

#### `set/get Router`

## Advanced

#### `import`

#### `commit`

#### `contextify`

#### `Version`

#### `Status`

-----
Page Url: src/reference/aeppic/querying.md
-----
# Querying

Queries are defined using [fluent api](#fluent-query-builder) or as a string that is being [parsed](#query-string) into a query graph. 

The best way to build a graph is to use the [fluent query builder](#fluent-query-builder), but for some simple queries the
string query method works pretty well even it does not expose all features.

For example using:

``` 
Aeppic.Query.children('my-root').form('folder').where('data.name').is('a*')
```

which returns all children documents of form 'folder' under 'my-root' where the field 'data.name' matches the suffix wildcard query 'a*'

Alternatively, though a little slower due to the parsing overhead, one can define the same query as a [string][StringQuerySyntax].

e.g.

```
"p:my-root f.id:folder data.name:a*"
```

Both queries get processed into a [parameterized query graph](#ExampleGraphSimple) .

> Since all documents have a specific form version associated with it and are stored in a tree of documents it is strongly recommended to limit queries to a form or limited list of forms and a sub section of a tree as in children or descendants of another document. 

Queries are then passed into e.g. the `Aeppic.find` function. 

e.g.

```
const q = Aeppic.Query.global().where('data.name').is('xox')

const documents = await Aeppic.find(q)
```

## Fluent Queries <a id="fluent-query-builder"></a>

The fluent query builder makes building queries simpler than creating the raw graph structure and makes it easier to write more complicated queries, especially as braces

The entry point to the fluent API is a `QueryScopes` instance which is accessible by
`Aeppic.Query`.

QueryScopes exposes the following scopes that can be used

  - `global()`
  - `children()`
  - `descendants()`

In contextified Aeppic instances such as `Design` controllers `children` and `descendants` are already preconfigured to use the document of the design as root of the query, but an alternate can be provided. To `Aeppic.Query.children()` inside a design controller for example would return all children of the current document, whereas `Aeppic.Query.children('apps')` could be used to get other children.

Let's use that as in the example:

`Aeppic.Query.children('my-root').form('folder').where('data.name').is('a*')`

The scope is usually followed by a form or multiple forms (e.g `.form('folder')` or `forms(['folder', 'file']`) which apply to all sub-conditions in the query.

After that the query can be built as a combination of `where(fieldPath).is(value)` queries potentially separated by `or` and nesting via `subQuery`.

### Scopes
-----
Page Url: src/reference/aeppic/modules/features.md
-----
# Aeppic.Features

In order to control certain implementation features within aeppic we employ [feature flags](https://en.wikipedia.org/wiki/Feature_toggle).

Each feature flag currently available is listed under `features` in the [Default Configuration](../../../reference/aeppic-server/005-server-configuration.md#features). 

::: info
Features are loaded initially after login or when not logged in but a connection to the server is configured. They are reloaded on every login/logout.
:::

## load()

Load or refresh all features flags from the server. It uses the `/api/v4/features/aeppic` endpoint by default and stores
the last loaded values in local storage which it falls back to in case it encounters errors reading from the server.

## isEnabled(name)

This checks whether a feature is enabled. It also logs a warning if that feature is marked as deprecated.

```js
const canUseFeature = Aeppic.Features.isEnabled('feature-tag-name') 
```

## use(name) {#use}

This checks whether a feature is enabled. It also throws an exception if the feature is not enabled and a warning if the feature is marked as deprecated.

```js
Aeppic.Features.use('feature-tag-name') 
```

## getOptionsFor(name)

This reads the configured options for a feature. It also marks a feature as being used (see [use](#use)), but doesn't throw if the feature is disabled.

```js
const options = Aeppic.Features.getOptionsFor('feature-tag-name') 
```
-----
Page Url: src/reference/aeppic/modules/functions.md
-----
# Aeppic.Functions

## resolveField

Resolve a field in a javascript object

`Aeppic.Functions.resolveField(object: object, pathToField: string): any`

```
const o = {
  data: {
    address: {
      street: 'Main Street'
    }
  }
}

const street = Aeppic.Functions.resolveField(o, 'data.address.street')
assert(street === 'Main Street')
```

## deriveChildId

Concatenate the string arguments into a valid identifier string.

`Aeppic.Function.deriveChildId(parentId: string, ...parts: string[]): string`

-----
Page Url: src/reference/aeppic/README.md
-----
# Overview

Interacting with aeppic is mostly done via the **Aeppic** interface.

It provides functions to create [documents](./document.md), edit them, query them etc.

```javascript
// Create a new document
const newDocument = await Aeppic.new(documentForm, parentDocument)
newDocument.data.name = 'Hallo'
Aeppic.save(newDocument)

// Delete a document (moving it into the Recycler)
Aeppic.delete('some-other-doc')

// Find documents
const docs = await Aeppic.find(Query.global().where('id').isOneOf(['a','b']))
const a = await Aeppic.get('doc-a')
const b = await Aeppic.findOne(Query.children().where('data.name').is('hello'))

const testDocs = await Aeppic.find('p:apps data.name:*test*')   
```

> See [Editable Document](./editable-document.md) for more info on editing a document

On top of that it also provides access to modules for handling more extended typical operations.


```javascript
// Access Authentication information
const auth = Aeppic.Auth
const currentAccount = Aeppic.Account
const homePage = Aeppic.HomePage

// Basic navigation
Aeppic.navigateTo('document-id')
Aeppic.navigateHome()

// Connection Info
const info = Aeppic.ConnectionStatus

// Access luxon via Aeppic.DateTime
const now = Aeppic.DateTime.local()

// Execute commands against the aeppic data model
const result = await Aeppic.Commands.execute('command-id', 'target-document')
```-----
Page Url: src/reference/tools/README.md
-----
# Tools

aeppic includes a few tools to get started.

## @aeppic/install-server {#install-server}

The install-server tool can be used to install aeppic-server from a [repository
server](#install-repository-server) like https://get.aeppic.com or update the base dataset.

This is mostly used for standalone installations via ansible or manually. Alternatively docker images are located at [registry.dev.curasystems.com](https://dev.curasystems.com/aeppic/aeppic-full/container_registry).

::: details README.md
<<< @../../../tools/install-server/README.md
:::

::: details Usage
<<< @/usages/install-server.txt{1} [Usage]
:::

## @aeppic/install-repository-server {#install-repository-server}

The install-repository-server tool can be used to install a repository server
to be used by [@aeppic/install-server](#install-server).

::: details README.md
<<< @../../../tools/install-repository-server/README.md
:::

::: details Usage
<<< @/usages/install-repository-server.txt{1} [Usage]
:::-----
Page Url: src/client/002 - interface.md
-----
# Available Interfaces

When a webpage is opened in the client it exposes some functions onto `window`.

::: tip
The features thus exposed are typically handled by `Aeppic`, but
in case of editing conflicts some functions might need to be called
directly.

E.g file editing is exposed via `Aeppic.editFile`. See [Aeppic File Handling](../reference/aeppic/aeppic#file-handling)
:::

The entire interface is made available on the property `host``:

::: details AeppicClientHostInterface (aeppic.ts)
<<< @../../../libraries/aeppic/src/aeppic/aeppic.ts#AeppicClientHostInterface
:::

```typescript
window.host = {
  type: 'aeppic-client',
  version: '1.3.7', // A semver compatible version
  /// A simple match function to test
  /// the expected semver e.g. '1.2.3' or '^1.2.3', or '^1.2' 
  /// against the actual version.
  match(expectedSemVer: string) -> bool { ... },
  ...
  // More see below
}
```


## Features

Flags on which features are supported can be checked. These are grouped by exposed
group.

```typescript
window.host.features = {
  files: {
    open: boolean
    edit: boolean
  }
}
```

## Files

```typescript
window.host.files = {
  /// Open a file in the local OS application associated with 
  /// the filetype.
  ///
  /// url: Where to download from 
  ///      When already downloaded before and ETag was returned it is used to NOT 
  ///      download again. (Stored in etag.txt next to the downloaded file)
  ///
  /// fileName: The name the file should get in the local filesystem
  ///
  /// {
  ///          id: Id of the aeppic document the file is associated with
  ///           v: The version of the aeppic document (only for logging etc) 
  ///   fieldName: The name of the field the file is store in
  ///    mimeType: The mime type of the file. If filename is not defined
  ///              this is used for a default extension if possible. 
  /// }
  ///
  /// Note: The file is marked `readonly` when possible
  ///
  async openFile(url, fileName, { id, v, fieldName, mimeType } = {}) {
      // Get an access token for the current user to hand to the 
      const accessToken = await window.host.helpers.readAccessToken(['content.read'])
      // Call the host
      ...
  },
  /// Open a file in the local OS application associated with 
  /// the filetype.
  ///
  /// url: Where to download from 
  ///      When already downloaded before and ETag was returned it is used to NOT 
  ///      download again. (Stored in etag.txt next to the downloaded file)
  ///
  /// fileName: The name the file should get in the local filesystem
  ///
  /// {
  ///          id: Id of the aeppic document the file is associated with
  ///           v: The version of the aeppic document (only for logging etc) 
  ///   fieldName: The name of the field the file is store in
  ///    mimeType: The mime type of the file. If filename is not defined
  ///              this is used for a default extension if possible. 
  ///     dataUrl: The EXACT identify url identifying the content. Important
  ///              for collision control. When NOT defined any collisions
  ///              are ignored.
  ///
  ///   Optional: Fields to add to the form encoded upload body. A new and unique dataUrl
  ///             is auto included.
  ///
  ///   uploadFormFields: { <FIELD>:<CONTENT-STRING> } 
  ///             
  ///   uploadFileFieldName: The name the binary file content is expected in for the upload
  ///   uploadUrl: The url to upload the file to   
  /// }
  ///
  /// Note: The file is marked as NOT `readonly` when possible
  ///
  async editFile(url, fileName, { id, v, fieldName, dataUrl, mimeType, uploadFormFields = {}, uploadFileFieldName = 'file', uploadUrl = '/api/_upload' } = {}) {
      // Get an access token for the current user to hand to the 
      const accessToken = await window.host.helpers.readAccessToken(['content.read', 'content.write'])
      // Call the host
      ...
  },
  /// Events to inform what happens in the host. E.g change events, edit events etc.
  subscribeToEditEvents(cb) {
      // Start listening for events
      ... calling back whenever an edit events happens /..
      cb(editInfo)
  },
  // Delete the locally downloaded document matching the document id with the binary 
  // stored in the specified field. The 'v' field is only used for logging purposes.
  async delete({ id, v, fieldName }) {
    ...
  },
  // Delete all files downloaded longer than `maxAgeInSeconds` ago.
  // NOTE: `Downloaded` NOT `modified`
  async deleteOld(maxAgeInSeconds = 60 * 60 * 24 * 7) {
    ...
  },
  // Delete ALL locally downloaded content
  async deleteAll() {
    ...
  },
  async listWatched() {
    ...
  },

}
```

### Errors

`openFile` and `editFile` can throw an exception object.

The error objects thrown are all structured in a similar fashion.

They have ONE key which is named after the error type and then the data fields associated with it.

e.g

```javascript
{
  DataUrlOutOfSync: {
    data_url_specified: "aeppic-local://client/98b70521-8617-4fcf-ab4b-97509f579013",
    last_known_data_url: "aeppic-local://client/98b70521-8617-4fcf-ab4b-97509f579013dasdasd"
  }
}
```

#### DataUrlOutOfSync

When a document was locally edited this will happen. The client will have generated a new data
url and thus detect a potential loss of data.

```javascript
{
  DataUrlOutOfSync: {
    data_url_specified: "aeppic-local://client/98b70521-8617-4fcf-ab4b-97509f579013",
    last_known_data_url: "aeppic-local://client/98b70521-8617-4fcf-ab4b-97509f579013dasdasd"
  }
}
```

#### FileAlreadyDownloading

Still downloading. Wait until finished

```javascript
{
  FileAlreadyDownloading: {
    path: "/local/file/path.txt",
  }
}
```

#### WatchFile

Error trying to watch changes to the file

```javascript
{
  WatchFile: {
    path: "/local/file/path.txt",
  }
}
```

#### CannotSendDownloadRequest

Could not connect to the server

```javascript
{
  CannotSendDownloadRequest: {
    info: { 
      url: string,
      dataUrl: string,
      accessToken?: string,
      explicitFileName: string,
      mimeType: string,
      document?: { 
        id: string,
        v: string,
        text?: string,
      },
      fieldName?: string,
      makeEditable?: boolean
    }
  },
  source: {
    message: string
  }
}
```

#### CannotDownloadFile

Could not download the file successfully

```javascript
{
  CannotSendDownloadRequest: {
    info: { 
      url: string,
      dataUrl: string,
      accessToken?: string,
      explicitFileName: string,
      mimeType: string,
      document?: { 
        id: string,
        v: string,
        text?: string,
      },
      fieldName?: string,
      makeEditable?: boolean
    }
  },
  status_code: number
}
```

#### CannotCreateLocalFile

Could not store the file locally successfully

```javascript
{
  CannotSendDownloadRequest: {
    info: { 
      url: string,
      dataUrl: string,
      accessToken?: string,
      explicitFileName: string,
      mimeType: string,
      document?: { 
        id: string,
        v: string,
        text?: string,
      },
      fieldName?: string,
      makeEditable?: boolean
    }
  },
   source: {
    message: string
  }
}
```

#### CannotReadFromDownload

Could not read from the response stream 

```javascript
{
  CannotReadFromDownload: {
    info: { 
      url: string,
      dataUrl: string,
      accessToken?: string,
      explicitFileName: string,
      mimeType: string,
      document?: { 
        id: string,
        v: string,
        text?: string,
      },
      fieldName?: string,
      makeEditable?: boolean
    }
  },
   source: {
    message: string
  }
}
```


#### CannotWriteLocalFile

Could not write into the local file

```javascript
{
  CannotWriteLocalFile: {
    info: { 
      url: string,
      dataUrl: string,
      accessToken?: string,
      explicitFileName: string,
      mimeType: string,
      document?: { 
        id: string,
        v: string,
        text?: string,
      },
      fieldName?: string,
      makeEditable?: boolean
    }
  },
   source: {
    message: string
  }
}
```

### Events

The events raised:

#### Change

When a file change is detected

```typescript
{
  Changed: [
    string, // The path of the file changed locally
    {
      url: string,
      dataUrl: string,
      explicitFileName: string,
      document?: { 
        id: string,
        v: string,
        text?: string,
      },
      fieldName?: string,
    }
  ]
}
```

#### Upload

All upload events have the same data associated with them. 

```typescript
type UploadFileInfo = {
  uploadKey: string,
  filePath: string,
  targetUrl: string,
  document: { 
    id: string,
    v: string,
    text?: string,
  },
  fieldName: string // Name of the field the document stores this binary in
}
```

So for `Started` it would look like this:

```typescript
{
  Upload: {
    Started: UploadFileInfo
  }
}
```

All events:

|Event Name|Description|
|-|-|
|Started|The upload has begun|
|Canceling|The upload is currently being canceled|
|Canceled|The upload was canceleld|
|Finished|The upload was successfully uploaded|
|Failed|The upload failed to upload|

## Viewer

This is an interface for multi-monitor support for specific use-cases. At the
moment it is meant for hooking up a DICOM viewer, but will be made more generi
over time.

```typescript
window.host.viewer = {
  isAvailable: false,
  url: '',
  numberOfMonitors: 0,
}
```

If available (if compiled like that)

```typescript
window.host.viewer = {
  isAvailable: true,
  getMonitors: async function() {
  },
  // Open all viewer windows (using the url as configured during compile time)
  openWindows: function() {
  },
  // Close them
  closeWindows: function() {
  },
  // Show the data on the monitor 0
  showOnPrimary: function(args) {
  },
  // Show the data on the monitor 1
  showOnSecondary: function(args) {
  },
  // Show the data on the monitor auxMonitorIndx
  showOnAuxiliary: function(auxMonitorIndex, args) {
  },
}
```

## Helpers (Internal)

```typescript
window.host.helpers = {
  // These are for internal use inside the previous modules and can be changed at any time between any versions
}
```
-----
Page Url: src/client/004 - Files Architecture.md
-----
# Files

Editing and working with local files is very important for businesses. The host applicationn
provides a number of functions to support editing these with local applications as seen
in the interface.

## Architecture

For most users the internal architecture is not important, but here is a description 
of the internal processes used for a deeper understanding. 

```mermaid
sequenceDiagram
  participant S as Aeppic Server
  participant A as Aeppic Web Application
  participant H as Host Desktop Client App
  participant OS as Operating System

  H->>A: hosts
  A->>S: accesses
  H->>OS: opens files using
  OS->>H: notifies changes
  H->>S: uploads changed file content
  H->>A: reports uploaded content
  A->>S: updates document to use new content 
```

## Edit Sequence

### Edit File command

```mermaid
sequenceDiagram
    participant AeppicApplication as Aeppic Application
    participant Client as Desktop App
    participant LocalStorage as Local Filesystem
    participant Server as Aeppic Server
    participant Editor as Editing Application

    AeppicApplication->>Client: Edit file
    Client->>LocalStorage: Check if file is downloaded for editing
    alt File not downloaded
        Client->>LocalStorage: Store data_url in data-url.txt
        Server->>Client: Download from Server
        Client->>LocalStorage: Store file, data_url, and ETag
        Client->>Editor: Open file for editing
    else File exists
        alt File is being uploaded
          Client->>AeppicApplication: Do not do anything as we are uploading
        else
          LocalStorage->>Client: Read data_url in data-url.txt
          Client->>Client: Compare data_url.
          alt no previous data_url (potentially aborted download)
            LocalStorage->>Client: Read ETag
            Server->>Client: Download from Server providing ETag. Do not redownload if same
          else data_url provided is different
            Client->>AeppicApplication: Do not do anything as we still have different data. `DataUrlOutOfSync` Error.
            note left of Client: Use openFile (without dataUrl) to see the most recent version
            note left of Client: Use returned dataUrl to ensure only that or derived is shown
            note left of Client: Use `delete` to remove the local copy.
          else NO data_url provided by Aeppic Application
            Client->>Editor: Open file for editing to overwrite the server version
          end
          Client->>Editor: Open file for editing
        end
    end
```

### File Change Sequence

```mermaid
sequenceDiagram
    participant AeppicApplication as Aeppic Application
    participant Client as Desktop App
    participant LocalStorage as Local Filesystem
    participant Server as Aeppic Server
    participant Editor as Editing Application

    LocalStorage->>Client: File Change Detected
    Client->>Client: Mark File as being uploaded
    Client->>Client: Generate new data_url
    Client->>LocalStorage: Delete ETag file and store data_url
    Client->>LocalStorage: Store upload marker file
    Client->>Server: Start Upload
    alt Upload goes through 
      Client-->>AeppicApplication: Send Event to notify of upload with new data_url
      Client->>Client: Wait for Upload to finish (This is done asynchronously)
      Client-->>AeppicApplication: Send Event to notify of upload finish with new data_url
      AeppicApplication->>Client: Aeppic Application acknowledges the new data_url for the document
      Client->>LocalStorage: Delete upload marker file
    else 
      Client-->>AeppicApplication: Send Event to notify of error during upload
      LocalStorage->>Client: Read data_url to use
      AeppicApplication->>Client: Send Retry to retry the upload
      note right of Client: Start Sequence with "Start Upload" as described above
    end
```

### File Change during upload

When a file change is detected **during** an upload, the upload is immediately cancelled. Once the upload
has been cancelled, the normal file change sequence as described above is executed.

### Resume Sequence

> [!CAUTION]
> This is not implemented yet.

When the system starts, the aeppic Application can ask the client to resume any uploads.
A list can be generated for all files where the upload was interrupted by looking
for the upload marker files. Each file can then be resumed which will start watching the files
for changed (not opening any editor though) and the file is being uploaded.

If edit is called (instead of resume) the call would be ignored, therefore it is important
for the client application to read the pending upload list and ask the user about what to do.

The pending file upload could be cancelled and the data_url overwritten with the latest known version.
The actual file can then be opened for editing again (it would still have any changes made since downloading).


-----
Page Url: src/client/001 - intro.md
-----
# Introduction

**Aeppic Client** is a desktop application designed to access our web application with enhanced stability and security. Compatible with both Windows and macOS, it utilizes the operating system's embedded browser engine. This integration ensures that aeppic Client is always as up-to-date as the latest OS patches, offering a fast and stable experience.

## Key Features

### Optimal Stability and Security
aeppic Client brings stability and security to your web application experience. By using the embedded browser engine of the OS, it remains consistently updated with the latest security patches. This design choice not only enhances performance but also maintains high security standards.

### Enhanced Privacy and Protection from Browser Extensions
One of the key advantages of our Desktop App is its ability to shield users from the potential risks posed by browser extensions. While traditional browsers can be vulnerable to malicious or snooping browser extensions, our Desktop App operates independently, reducing the risk of third-party extensions compromising your data or monitoring your online activities. Your privacy and security are paramount, and our Desktop App ensures a safer and more secure browsing environment, free from the concerns associated with potentially harmful extensions.

### Customizable Browser Engine
Understanding the importance of operational continuity, aeppic Client allows for the locking down of the actual browser engine in scenarios where OS updates may interfere with critical business applications. This feature is invaluable for maintaining the integrity of essential business operations. 

### Advanced File Interaction
One of the unique capabilities of aeppic Client is its ability to open web-stored files directly in Microsoft Word and monitor them for changes. This functionality extends beyond the limitations of standard browsers, offering a more integrated and efficient workflow.

### Independent Caching for Enhanced Security
aeppic Client employs a separate caching system, distinct from your regular web browsers. This approach significantly reduces the risk of security token leakage or other sensitive data exposure, ensuring that your browsing on aeppic Client remains secure and private.

## Installation
aeppic Client can be easily installed on both Windows and macOS systems. Follow the standard installation process for your respective operating system to get started with a more secure, stable, and efficient web application experience.

-----
Page Url: src/client/003 - Conflicts.md
-----
# Handling conflicts

When a file is edited, the change is detected and uploaded.
Each upload means a new dataUrl is generated which uniquely
identifies the binary content on the server.

> See [Files Architecture](./004%20-%20Files%20Architecture.md) for sequence diagrams.

After that is completed an event is raised to the hosted
aeppic application which usually would edit the instance
of the aeppic document in the correct file field and save it.

But it is possible, that the another person was also editing the
same file on another machine/client and this would cause a loss
of data when calling editFile again.

editFile will raise an exception when this is detected and
the error will include the expected dataUrl.

```
{
  DataUrlOutOfSync: {
    data_url_specified: "aeppic-local://client/98b70521-8617-4fcf-ab4b-97509f579013",
    last_known_data_url: "aeppic-local://client/98b70521-8617-4fcf-ab4b-97509f579013dasdasd"
  }
}
```

In this case either:

1. delete the local file (`deleteFile`) and call it again.
2. call `openFile` / `editFile` without a dataUrl or with the correct `last_known_data_url`) to allow the user to look at the local file first.

-----
Page Url: src/index.md
-----
---
layout: home
title: Aeppic Documentation
hero:
  name: aeppic
  text: Modern Business Applications
  tagline: Reliable, Extensible, Traceable 
  image:
    src: /images/header.png
    alt: aeppic
  actions:
    - text: Get Started
      link: /guide/001-getting-started.md
      theme: brand
    - text: Concepts
      link: /concepts/README.md
      theme: alt
    - text: Examples
      link: /examples/README.md
      theme: alt
features:
  - title: Form Based
    details: All data and configuration is contained in an hierarchy of documents based on forms. Forms are written in a text based (markdown) format for future proofing and versioning.  
  - title: Rules
    details: Every change made results in an auditable event. Events can be used via If-Then triggers to automate certain other actions.
  - title: Services
    details: Custom services can interact with the server api directly and provide features or subscribe to events.
  - title: Workflows
    details: Stamps marking state changes are integrated deeply and enable easy to understand business workflows. Stamps contain cryptographic hashes which can link a state change with a user's biometric information. 
  - title: Security
    details: An easy to understand and powerful lock & key security architecture is built-ina and authentication can optionally be linked to OAuth providers like Azure AD,keycloack,etc. for single-sign-on
  - title: Modern Web-Design
    details: Enjoy the benefits of having access to all features modern web-design supports. A good generic design for all forms is included, but can be easily adjusted by developers and designers.
  - title: Scalable
    details: Scale from the very small to the very big based on a hierarchical data structure.
  - title: Open-Data Format
    details: A business application supports a business. As such all data and configuration must remain as open for the business as possible to get the maximum out of it. 
  - title: Built-to-Last
    details: The core design of aeppic is to survive the continuous and luckily relentless drive towards better technology. Business rules and content must remain serviceable for as long as possible. 
---
-----
Page Url: src/architecture/Rendering/010-dynamic-components.md
-----
# Dynamic Components

Most of the HTML generated for the client is rendered using `Dynamic Components`. A dynamic component is
stored as a document in aeppic and gets converted into the appropriate javascript component to generate
the necessary HTML.

## Types

| Name                 | Scope                                                                                             | Comments                                             |
| -------------------- | ------------------------------------------------------------------------------------------------- | ---------------------------------------------------- |
| Design               | Render a document                                                                                 | `title` design to render a document **as a** 'title' |
| Control (Field)      | Render the input/output of a specific field (or more) of a document                               | `text` to render a text input field                  |
| Control (Standalone) | Render something with access to an editable document without explicitely being bound to one field | `comments` to allow commenting bound to a document   |
| Layout               | Free component to render arbitrary content                                                        | Used for the application root layout or fields       |

::: tip
Rendering of forms makes use of many dynamic components which is in described [here](./020-forms.md).
:::

### Design

Designs render documents in order to show them in context of other document. Mostly this is 

### Layout

Layout is a very versatile dynamic component and should be used lightly only when it makes sense.

It is for example used during [Form rendering](./020-forms.md) to allow adjusting the style of forms without loosing the benefits
of an automatically generated form.

It is also used for the entry point of the entire html page with the root layout which is referenced in the `root` document. It is
the first thing being rendered and it decides what to do. 

> NOTE: Routes set on `root` influence which layout gets rendered initially.

## Rendering

Whenever e.g. an `ae-layout` is used the `ae-layout-selector` component goes ahead and asks `Aeppic.DynamicComponents` to 
ensure the component is available and what the component's id is. This gets then used with a `<component :is="..." ...>` with
all attributes and parameters forwarded to it. 

Each dynamic component has at least four inputs into the rendering process taken from the respective component's document

| Type            | Description                                                                            |
| --------------- | -------------------------------------------------------------------------------------- |
| HTML Template   | The HTML template is parsed using [vue](https://vuejs.org). Currently using v2 of vue. |
| CSS Stylesheet  | The CSS stylesheet is parsed via less and injected into the page                       |
| Controller Code | The controller code is slightly reformatted and injected into the vue template         |
| Packages        | Packages are packaged dependencies which provide components and/or functions to use    |

  


-----
Page Url: src/architecture/Rendering/020-forms.md
-----
# Form rendering

::: warning
This documentation describes asperationally how aeppic <Badge type="tip" text="4.x" vertical="middle"/> renders forms.

In the preceding version <Badge type="tip" text="3.x" vertical="middle"/> this implementation of ae-form is only enabled as an opt-in feature
for forms that opt-into rendering with this method. It is still experimental and the internals
might change until release
::: 

::: tip
Opt-In

The new rendering behavior as documented below is activated with `[feature flag](/reference/aeppic/Modules/features.md)`
named 'aeppic.forms.rendering.experimental'

Forms also need to contain the tag `experimental-rendering`.
::: 

## Components

Displaying a form in the UI is done via the `ae-form` component. 

```html
<ae-form :document="locals.documentToDisplay" ...></ae-form>
```

> There a a number of parameters that can be passed into `ae-form` these are more fully documented in `[ae-form](/reference/aeppic/forms/ae-form.md)`

When called to display a document `ae-form` looks up any information related to the form of the document passed in.

When a form definition gets parsed it outputs the following hierarchy:

`Form` / `Root Section` / `Header` + `Paragraph` + `Sub Sections`

### Conceptual DOM tree

```mermaid
flowchart LR
  subgraph ae-form
  ae-form-section
  end
  subgraph ae-form-section
  ae-header
  ae-form-paragraph
  ae-form-section_SubSection
  end
  subgraph ae-form-paragraph["ae-form-paragraph"]
  text
  field
  control
  end
  subgraph ae-form-section_SubSection["ae-form-section"]
  end
```

### Full Component Tree

The full tree is a little more complex as the actual html is user configurable on multiple levels.

- The components `ae-form`, `ae-form-section`, and `ae-field` use a configurable layout and can be changed via a `Theme`.
- `ae-control` is a [dynamic component](./010-dynamic-components.md) itself. When used standalone and as used by `ae-field`.
- `ae-form-parapgraph` is a helper component to be used inside of section layouts to render the paragraphs.
  It provides [slots](https://v2.vuejs.org/v2/guide/components-slots.html) for customization.
- `ae-header` is a simple component which is also used by section layouts to help with rendering a sections title header.
 
```mermaid
flowchart TB
  subgraph ae-form
  ae-layout_Form
  end
  subgraph ae-layout_Form["ae-layout"]
  ae-form-section
  end
  subgraph ae-form-section
  ae-layout_Section
  end
  subgraph ae-layout_Section["ae-layout"]
  ae-header
  ae-form-paragraph
  ae-form-section_SubSection
  end
  subgraph ae-form-section_SubSection["ae-form-section"]
  end
```
  
```mermaid
flowchart TB
  subgraph ae-form-paragraph 
  html
  ae-field
  ae-control_Standalone
  end
  subgraph ae-field
  ae-layout_Field    
  end
  subgraph ae-layout_Field["ae-layout"]
  ae-control
  end
  subgraph ae-control_Standalone["ae-control (Standalone)"]
  end
```

> Standalone controls not associated with a field in the document would be on the level of the ae-field and not wrapped.

The layout components are defined at runtime as aeppic documents and thus extremely flexible. A form layout could thus forgoe
all rendering as outlined above and just display `<h1>Hello World</h1>` and be done. This wouldn't render any fields, but allowed.

## Responsibilites

Each component has a specific responsibility

| Component                        |                                                                                                                                      |
| -------------------------------- | ------------------------------------------------------------------------------------------------------------------------------------ |
| ae-form                          | Lookup up form information, including which layout to use and render with those parameters starting at the root **section**          |
| ae-layout (*inside the form*)    | Use the form information to setup the main style scope (CSS) and render the chosen root section (pass along any required parameters) |
| ae-form-section                  | Lookup the parts of the section (fields,sub-sections) and pass it to the section layout                                              |
| ae-layout (*inside the section*) | Layout the parts according their type, order, and designated styles                                                                  |
| ae-field                         | Lookup the layout for the field and control component to render the field with and render it.                                        |
| ae-layout (*inside the field*)   | Layout the field surroundings (label,help,validation,etc) if the control cannot do it then position the control                      |
| ae-control                       | Render the actual user interactive html to let the user change a field in the editable docuemnt (unless it's set as readonly)        |

### ae-form

1. Look up the document's form information `Aeppic.Forms.getRenderInfoForDocument`
2. Lookup the form's theme to be used.
3. Ensure the editable document is registed in an existing outer edit context
4. Figure out which section to render. Each form consists of a tree of sections, thus the root section is normally
   the one being rendered, but this could have been overriden.
5. Instantiate the layout based on that information and render the section `<ae-layout :layout-id="..." :params="..."></ae-layout>`

### ae-layout (*inside the form*)

A layout is dynamic content, thus this description only shows what a form layout needs to do for *normal* rendering to occur. 
 
> There is often no need to override the form layout as it makes more sense to override the 
> section styles and/or respective layouts.

1. Render the root section to fill the entire area

```html
<ae-form-section :section="params.section"
            :param-sections-params="params.sectionParams"
            :class="params.sectionClass"
            :style="params.sectionStyle"
            >
</ae-form-section>
```

### ae-form-section

1. Lookup the parts in the form making up the section
2. Lookup the layout to use for the section
3. Render the layout and pass in the inherited parameters and the parts/styles discovered

```html
<ae-layout :layout-id="layoutId"
           :params="layoutParams"
           :class="layoutClass"
           :style="layoutStyle"
           >
</ae-layout>
```

## ae-layout (*inside the section*)

1. Define the layout of the different parts to be rendered in the section

Example:

```html

```

## ae-form-paragraph

Each part making up the section requires some basic layouting logic. This is embedded with a default rendering rule. Each rule 
is overridable using slots from the section layout.

### Part Types

| Name                          | Description                                                                    |
| ----------------------------- | ------------------------------------------------------------------------------ |
| HTML                          | Pure HTML/Markdown                                                             |
| SECTIONS                      | A list of sibling sections                                                     |
| FIELD                         | A single field                                                                 |
| LINE_OF_FIELDS                | A horizontal list of fields                                                    |
| FIELDS_WITH_TEXT              | A block of markdown with embedded fields with                                  |
| STANDALONE_CONTROL            | A standalone control                                                           |
| LINE_OF_CONTROLS              | A horizontal list of standalone controls                                       |
| STANDALONE_CONTROLS_WITH_TEXT | A horizontal list of standalone controls with text/html around or between them |

## ae-field

A field is responsible for allowing read/write access to a documents field. 


> A field currently has access to the entire document, but this will become an opt-in behavior in an upcoming release

## ae-layout (*inside the section*)

1. Define the layout of the different parts to be rendered in the section
   
## ae-control


-----
Page Url: src/architecture/Rendering/README.md
-----
# Overview

## -----
Page Url: src/architecture/Data Model/README.md
-----
# Data Model

The Data Model underlying aeppic is a tree of documents.

The top nodes that always exist are

```mermaid
graph LR
  /((root)) --> apps
  /((root)) --> recycler
  /((root)) --> system
  /((root)) --> preferences
```
-----
Page Url: src/architecture/Other Features/README.md
-----
# Other Features

[[toc]]


## Staging

![Staging](/images/feature-staging.png){ width=256 }

During multiple phases of using aeppic it is helpful to setup a well environment. With specific data, or a new version of the aeppic server without interfering with the active deployment.  

In aeppic this feature is called **Staging** and it allows to start a new server instance accessing the same data (with some visibility policies applied). Changes done inside the staged environment are NOT visible or in any way transported back to the production deployment.

::: details Prerequisites
- The server must be reachable by subdomains
- If https is used the subdomains must have valid certificates
- The server must run in an environment with [podman](https://podman.io/)
- The server must be able to access a repository server if staging requests a different aeppic-server version
- The server must have enough resources (CPU, RAM, Storage) to run the stages. 
:::

::: tip Alternatives
In a full cloud setup staging can be manually accomplished some of the abilities of staging can be handled by fully cloning a machine and/or syncing data back, but even in this case it will be handy to boot with limited data visibility or stopping the import at a fixed point in time. 
:::

### Example Use Cases

- During development, to run business rules, workflows, commands, custom actions, etc. against reproducible data
- During regression testing, to validate that updated designs or code still results in the same behavior
- In production, to have a point-in-time look at an entire system in the past
- For auditing, to have a point-in-time look at a system, but with certain non-audit relevant data removed

### Architecture

The aeppic api has an interface under `/v4/staging/` to inspect and interact with the stages. 

-----
Page Url: src/architecture/Caching/client.md
-----
# Client-Side Caching

The web-client application requires to load all relevant operational model data from the server. Since the [Data Model](../../architecture/Data%20Model/README.md) is hierarchical, the caching strategy makes use of that. Client-Side Caching is an essential part off [Offline Support](../Offline/README.md).

It is used to implement [Offline Mode](../Offline/README.md), but also to speed up server interactions.

## Data Assets

### Example

To instruct aeppic to cache all documents located in `system` we can add
the following policy to the cache.

```ts
Aeppic.Cache.addPolicy({
  name: 'system',
  queries: Aeppic.Query.descendants('system'),
  refresh: 'onboot',
  strategy: 'stale-subscriptions',
  evict: 'never',
  reason: 'We need /system all the time :)',
})
```

This will add named policy which will load all descendants of system to the local application cache. 

The Cache Controller (which is exposed as `Aeppic.Cache`) loads any previously configured policies on boot.

The controller is actually running in a service worker and not on the UI thread. The Aeppic.Cache object allows
interacting with the shared controller. If the installed service worker does not expose a cache controller `Aeppic.Cache`
silently ignores all calls. 

The policy descriptions itself, are stored in local storage associated with the current login info.
-----
Page Url: src/architecture/Caching/server.md
-----
# Server-Side Caching

The server web app is holding a cache of recent queries in an [Arc](https://en.wikipedia.org/wiki/Cache_replacement_policies#Adaptive_replacement_cache_(ARC)) cache. 

Every time a document is created/modified/deleted it is checked against the server side cache and updates them as necessary. Queries that are affected by a document change are evicted from the cache by default.

The detection logic can be found here:

::: code-group
<<< @../../../server/server/server/cache.js#isQueryInfluencedByDocument{ts} [server/server/server/cache.js]
:::

> Note: It also checks the previous state of the document. So basically each cached query
> is checked against the previous state and the new state.

-----
Page Url: src/architecture/Caching/README.md
-----
# Caching

Caching is enormously helpful in getting an application feel snappy by reducing the load on underlying data sources. Nonetheless it is not without peril and has to be done with some care.

aeppic uses caching in a number of areas to improve performance.

- [Client-Side Caching](./client.md)
- [Server-Side Caching](./server.md)   -----
Page Url: src/architecture/Offline/README.md
-----
This documentation aims to provide a clear and well-structured guide to implementing offline support in Aeppic. To achieve this, we'll address two key components: static assets, data models, and authentication information.

Please note that Aeppic does not directly configure caches or service workers. It provides tools for these purposes, but the application itself needs to be configured to utilize them effectively.

Requirements for Offline Support
--------------------------------

To ensure compatibility with offline support, your browser must support the following features:

1. [IndexedDB](https://developer.mozilla.org/en-US/docs/Web/API/IndexedDB_API): Check compatibility [here](https://caniuse.com/indexeddb).
2. [Service Worker](https://developer.mozilla.org/en-US/docs/Web/API/Service_Worker_API): Check compatibility [here](https://caniuse.com/serviceworkers).

These features are supported by all modern evergreen browsers.

Static Assets
-------------

The `@aeppic/server` hosts all static assets, including the SPA HTML template and the API endpoint. To enable caching, these assets must be downloaded. While you can use any service worker to implement this, the Aeppic project includes one: `@aeppic/service-worker`.

The `@aeppic/service-worker` project handles client-side caching strategies for browsers. It operates alongside the `/api/v4/statics/_list` endpoint and the Statics Configuration. Setting this up is essential for offline support.

Data Model
----------

For offline functionality, the client needs to support all operations as if the server were available. This includes all CRUD operations, client-side business rules, and commands. The data model should accommodate these requirements, even when offline. To achieve this, the relevant data must be included in the cache.

Boot Sequence
-------------

During the initial startup of Aeppic in the browser, several processes occur, depending on how it's embedded in the HTML. By default, calling `Aeppic.renderLayout` renders the `ae-root` component into the designated DOM element. The system then retrieves documents from the server and performs queries. For a successful offline user experience, all requests must have been loaded into the data model cache.

Data Consistency
----------------

Maintaining data consistency is crucial for operational offline mode. The caching layer ensures a consistent view of the data model, even when online and subject to changes from connected clients. Local document changes are first applied locally and written to a change queue. The difference in offline mode is that the document and the change queue are written to IndexedDB when no server connectivity is available or when offline mode was switched on.

Security
--------

When configured for offline mode, Aeppic saves the last authenticated user information and assumes that user is still logged in. Logging off does not delete any pending changes by default (for data safety reasons), and these changes are not encrypted. If untrusted user scenarios are a concern, the application may implement an interactive user process before logging out to inform the user about potential data loss. If necessary, pending changes can then be deleted.
-----
Page Url: src/architecture/Queries/README.md
-----
# Querying

## How is `Aeppic.find(q)` processed ?

The actual processing of the queries depends somewhat on the backend used. Let's go through the case of the default in-memory datamodel in a slightly simplified fashion.

1. The query gets converted into a [`QueryMatcher`](#QueryMatcherCode)
2. The query gets converted into an iterator which can be used to efficiently iterate all known documents using the available indexes

The matching function needs to be efficient of course, so it compiles the graph
into an optimized testing function. Subsequent identical queries can re-use
the same function (as long as only the parameters are different) which is likely to be optimized into native code by the javascript engine.

Even though the QueryMatcher is quite fast, it is best not to call it on documents that cannot possibly match anyway. Thus [indexing](#IndexingCode) is used to iterate only over a superset of documents that need to be tested.

The [`Iterator`](#QueryFindingCode) is looking through the graph and calculates
which Index to iterate in which order to avoid iterating over all documents.

In order to do that, it takes each subpart in the query e.g. `f.id:X AND data.age:8` and checks whether those fields are covered by any index. If one field is covered, the Index
is asked to provide an iterator over that subset of documents. In case multiple terms are covered they are intersected (in the case of `AND`) with the smallest sets iterated first.

The end result is a enumeration of documents defining a possible superset of the requested
data which then get tested using the QueryMatcher.

> At the moment sort is done at the end as the iterators do not support sorted indexes

## Potential Optimizations

::: info
There are a number of obvious optimizations possible. Such as using the knowledge
of the schema and the data fields matched to limit searching. But none of those are 
implemented at the moment. The long-term goal is to move the data organization into
a dedicated database such as sqlite or equivalent.
:::

## String Queries

Queries made using string queries using a variant of the lucene query syntax get parsed into a simple graph by a [Parser](#QueryStringParserCode)

The parsing first uses and underlying raw PEG parser which emits a strict left/right syntax tree which then gets refined in order to join adjacent nodes into a more efficient structure (i.e. joining compatible adjacents terms into one condition) identical to what the fluent syntax would emit.

Some examples can be found in the [tests and source](#FluentQueryBuilderCode)

## Example Graphs

### <a name="ExampleGraphSimple"></a>Simple Query

```json
{
  graph: {
    operator: 'AND',
    conditions: [
      {
        field: 'p',
        fieldPrefix: null,
        term: { id: 0 },
        termQuotes: 'none',
        similarity: null,
        boost: null,
        termPrefix: null
      },
      {
        field: 'f.id',
        fieldPrefix: null,
        term: { id: 1 },
        termQuotes: 'none',
        similarity: null,
        boost: null,
        termPrefix: null
      },
      {
        field: 'data.name',
        fieldPrefix: null,
        term: { id: 2 },
        termQuotes: 'none',
        similarity: null,
        boost: null,
        termPrefix: null
      }
    ]
  },
  parameters: { '0': 'my-root', '1': 'folder', '2': 'a*' },
  ...
}
```

## Code
### Fluent Query Builder {#FluentQueryBuilderCode}

::: details query-fluent.spec.js and fluent.ts

::: code-group 
<<< @../../../libraries/aeppic/test/model/query-fluent.spec.js
<<< @../../../libraries/aeppic/src/model/query/fluent.ts
:::

###  Query String Parser {#QueryStringParserCode}

The initial form parser is defined in PEG and parsed by [PEG.js](https://pegjs.org)

::: details query-parser.peg
<<< @../../../libraries/query-parser/src/query-parser.peg
:::

The initial parser defined outputs a simple left/right graph which is post-processed by a query refiner.

::: details query-refined.spec.js and refining.ts

::: code-group
<<< @../../../libraries/aeppic/test/model/query-refiner.spec.js
<<< @../../../libraries/aeppic/src/model/query/refining.ts
:::

###  Query Matcher {#QueryMatcherCode}

Takes a query graph and allows to test documents against it to find out
whether a document matches the query or not.

::: details query-matcher.spec.js and match.ts

::: code-group
<<< @../../../libraries/aeppic/test/model/query-matcher.spec.cjs{js}
<<< @../../../libraries/aeppic/src/model/query/match.ts
:::

## Query Finding Algorithm {#QueryFindingCode}

Takes a query graph and allows to test documents against it to find out
whether a document matches the query or not.

::: details query-matcher.spec.cjs and finder.ts

::: code-group
<<< @../../../libraries/aeppic/test/model/query-matcher.spec.cjs{js}
<<< @../../../libraries/aeppic/src/model/query/finder.ts
:::

### Indexing {#IndexingCode}

Takes a query graph and allows to test documents against it to find out
whether a document matches the query or not.

::: details query-indexer.spec.spec.js and document-index.ts

::: code-group
<<< @../../../libraries/aeppic/src/model/indexing/document-index.ts
<<< @../../../libraries/aeppic/test/model/query-indexer.spec.cjs{js}
:::
-----
Page Url: src/architecture/README.md
-----
# Architecture

-----
Page Url: src/concepts/017 - Commands/README.md
-----
# Commands

An aeppic Command is a user initiated invocation of one or more Actions. They often query or change documents and can resolve by doing a navigation.

Commands are guarded by Keys and Locks.-----
Page Url: src/concepts/015 - Business Rules/README.md
-----
# Business Rule

A business rule in aeppic is an `If-Then` behavior which watches `Changes` and performs `Actions` in response.-----
Page Url: src/concepts/020 - Rendering/040-lists.md
-----
# Lists-----
Page Url: src/concepts/020 - Rendering/015-controls.md
-----
# Controls

Controls are the visual representation of parts of forms, usually associated with form fields.

They are **identified by name** which can include a namespace (colon separated as in `ae:my-example`).

Controls are documents of type `control-form` and consist of a template, style, and controller script.

When a field is defined in a form and no explicit control name is specified, the control name is derived from the field type. For example, a field of type `select` will use the control `select`.

## Usage

Controls are most often used inside the form definition (which is defined as FormDown). The control name is put into a styling directive in front of a fields label or fields definition. 

Styling directives are enclosed in curly braces `{}` and can contain multiple key-value pairs separated by `|`. The key-value pairs are separated by `=`. The key is the name of the parameter and the value is the value of the parameter. If the value contains spaces, it must be enclosed in quotes. Multiple styling directives can be put right after each other.

If the first key is not followed by a `=`, the value is assumed to be either a built-in styling tag or the control name to use.

```formdown
// A simple text field. This will use the control `text`
[Name][name]

// A number field. This will use the control `number`
[Age][age]
[age]: number

// The `hidden` directive is built-in and will hide the field. It is part of a small group of built-in directives.
{hidden} 
[Height][height]

// Render any occurrence of `height` as a number field with a unit of cm. This will use the control `number`.
// The only exception is any label that has explicitly overridden the control.
[height]: number <unit=cm>
```

// A standalone control. This will use the control `custom-control`
{custom-control age=true}

// Rendering a field with a different control. This will use the control `my:custom-control` where `my` must match the
// namespace of the control and `custom-control` is the name of the control.
{my:custom-control another-param="value"}
[Name][name]


### Built-In Styling Directives

The following built-in styling directives are available:

- `hidden` and `hide`: Hides the field.
- `show`: Shows the field (only makes sense in combination with `if`).
- `readonly`: Makes the field read-only.
- `editable`: Makes the field editable (only makes sense in combination with `if`).

The can be used with `if` to conditionally show or hide fields.

```formdown
[Show Field][showField]
[showField]: Switch

// A field that is only shown if the field `showField` is true
{show if showField}
[Another Field][anotherField]

[Show other Field][showOtherField]
[showOtherField]: Select
  - FIRST: First
  - SECOND: Second

// A field that is only shown if the field `showOtherField` is `FIRST`
{show if showOtherField=='FIRST'}
[Yet Another Field][yetAnotherField]
```

## Lookup Rules

As controls are identified by name, the lookup rules are are more complex than for layouts, but similar to [designs](020-designs.md#lookup-rules). 

Controls can be limited as to which types of fields they support, which cardinality, or standalone mode. These are checked when a control 
matching the namespace and name is found.

> The search inside control folders are always sorted by `created.at` date, so in case of multiple
> controls with the same name, the one created first is used.

The lookup rules are as follows:

  1. If the **form of the document** being rendered has a child document of type `control-form` with matching name (and namespace), then that document is used as the control.
  2. With the Form of the document as a starting point, we look for a folder on each level to each successive parent with the type `control-folder` and look for a child document of type `control-form` matching the controls namespace and name.
     1. If the current ancestor we investigate is a `form-folder`, we look through any referenced `control-folder` from the styling section of the folder.
     2. If the current ancestor we investigate is an `application-form`, we look through any referenced `control-folder` from the styling section of the application.
     3. If we reached the root we look through any referenced `control-folder` from the styling section of root.
  3. Then we lookup the control in the System Control folder (`controls-folder`) of the system folder
  4. If there is nothing found still we ignore the name of the control and use the default control for the field type from the `default` folder in the system control folder

### Lookup Graph

This flowchart visually represents the lookup rules for controls.

```mermaid
flowchart TB
  start("Look for control with namespace and name") --> lookForMatchingChild("Check for matching 'control-form' as child of the document's form")
  lookForMatchingChild -->|Found| useControl
  lookForMatchingChild -->|Not Found| checkNextAncestor
  checkNextAncestor("Look in next ancestor.") --> checkType("Check type of current ancestor")
  checkType -->|form-folder| checkInStylingSection
  checkType -->|application-form| checkInStylingSection
  checkType -->|root| checkInStylingSection
  checkType -->|unknown| checkNextAncestor
  checkType -->|none| lookInSystemControlFolder
  checkInStylingSection(Look at all 'control-folder's from styling section) --> |Found| useControl
  checkInStylingSection --> |Not Found| checkNextAncestor
  lookInSystemControlFolder("Look in system control folder") --> |Found| useControl
  lookInSystemControlFolder("Look in system control folder") --> |Not Found| lookInDefaultControls
  lookInDefaultControls("Look in `default` folder under the system controls folder") -->| Found| useControl
  lookInDefaultControls -->|Not Found| nonFound("End: No control found")
  useControl --> endControlFound("End: Control Found")
  endControlFound("End: Control Found")
```-----
Page Url: src/concepts/020 - Rendering/010-forms.md
-----
# Rendering forms

::: info
This describes the new v4 rendering process.
:::

Once a form is defined, any document based on it can be dynamically rendered via `<ae-form>`.

The component uses the structured form info provided by aeppic for that document to render it via a layout. The layout gets access to a view of the form which the layout uses to hierarchically render the form components.

The layout used at the moment is `6c39f2cc-6059-42b2-8796-5c803899367c` when the form is tagged with .

-----
Page Url: src/concepts/020 - Rendering/030-layouts.md
-----
# Layouts

## Purpose

## Example

-----
Page Url: src/concepts/020 - Rendering/020-designs.md
-----
# Designs

Designs are the visual representation of documents.

They are **identified by name** which can include a namespace (colon separated as in `ae:my-example`). The name signifies as what to render a document.

For example, a common design name is `title`, which renders a document **as a** title. Usually this means render just the content of the name field of the document. Another example is `card-content`, which renders a document as one would inside a card or `tree-item`.

Designs are documents of type `design-form` and consist of a template, style,and controller script.

## Usage

In order to use a design in html you would use the following syntax:

```html
<!-- Render a specific document as a title -->
<ae-design name="title" document-id="my-document-id"></ae-design>

<!-- Render a document as linked to by a reference field -->
<ae-design name="title" :document="data.myRefField"></ae-design>
```

### Parameters

Designs can have parameters. These are passed as attributes to the design element. For example:

```html 
<ae-design name="title" document-id="my-document-id" :param-my-param="true" param-another="Fixed Value"></ae-design>
```

These parameters are available in the controller script of the design and its html template under the object `params` where
each attribute is camel-cased. In the above example:

```javascript
// controller script (any function inside)
myFunction() {
  console.log(params.myParam); // true
  console.log(params.another); // Fixed Value
}
```

Or in the design template:

```html
<div v-if="params.myParam">
  <p>{{params.another}}</p>
</div>
```

## Lookup Rules

As designs are identified by name, the lookup rules are are more complex than for layouts, but similar to [controls](015-controls.md#lookup-rules). The lookup rules are as follows:

  1. If the form of the document being rendered has a child document of type `design-form` with the same name as the design, then that document is used as the design.
  2. With the Form of the document as a starting point, we look for a folder on each level to each successive parent with the type `design-folder` and look for a child document of type `design-form` with the same name as the design.
     1. If the current ancestor we investigate is a `form-folder` we look through any referenced `design-folder` from the styling section of the form.
     2. If the current ancestor we investigate is an `application-form` we look through any referenced `design-folder` from the styling section of the application.
     3. If we reached the root we investigate the root form and look through any referenced `design-folder` from the styling section of the root.
  3. At last we lookup the design in the Fallback Design folder (`id:fallback-designs`) of the system folder.

### Lookup Graph

This flowchart visually represents the lookup rules for designs.

```mermaid 
flowchart TB
  start("Look for design with namespace and name") --> checkDesignForm("Check for matching 'design-form' as child of the document's form")
  checkDesignForm -->|Found| useDesign
  checkDesignForm -->|Not Found| checkNextAncestor
  checkNextAncestor("Look in next ancestor.") --> checkType("Check type of current ancestor")
  checkType -->|form-folder| checkInStylingSection
  checkType -->|application-form| checkInStylingSection
  checkType -->|root| checkInStylingSection
  checkType -->|unknown| checkNextAncestor
  checkType -->|none| fallbackDesign
  checkInStylingSection(Look at all 'design-folder's from styling section) --> |Found| useDesign
  checkInStylingSection --> |Not Found| checkNextAncestor
  fallbackDesign("Look in system fallback design folder") -->|Found 'design-form'| useFallbackDesign("Use Fallback Design")
  fallbackDesign -->|Not Found| nonFound("End: No design found")
  useDesign --> endDesignApplied("End: Design Applied")
  useFallbackDesign --> endDesignApplied
```
-----
Page Url: src/concepts/020 - Rendering/README.md
-----
# Introduction to Rendering
-----
Page Url: src/concepts/010 - Documents/010-document data.md
-----
# Data

aeppic saves and handles json structured data with forms defining their schema.

## Binary Data

Binary data inside forms is store in fields of type `File`. They representa a binary 
data stream represented by an immutable URI which can be resolved to a binary data stream. 

Files data streams are uploaded via `Aeppic.Upload` and the server api will put it into the
upload store initially.-----
Page Url: src/concepts/010 - Documents/README.md
-----
# What is a document ?

An aeppic document is the most basic data structure of the aeppic [Data Model](../../architecture/Data%20Model/README.md). All data, most of the configuration, and most of the rendering behavior is stored in aeppic documents.

They are organized in a tree of documents starting at the `root`. It is explained in more detail in [Data Model](../../architecture/Data%20Model/README.md).

Apart from necessary metadata, all possible user content of a document is declared in an associated [Form document](./005-forms.md).
-----
Page Url: src/concepts/010 - Documents/005-forms.md
-----
# Forms

![](/images/feature-form.png){width=256}

An aeppic form is a document describing the possible content (a schema) for documents based on that form. It is mostly derived from the form's definition which is written in pure text in `formdown`. 

`formdown` is a dialect of [markdown](https://www.markdownguide.org/) which is defined for the purpose of declaring forms as easily as possible.

::: info Why a text based format ?
We chose to use text for forms as it makes versioning and understanding very easy. Visual form editors can be written on top of it though if needed. It also better aligns with the desire to keep data very open and not hidden in very technical formats.
:::

## Fields

A form is normal markdown that also supports **fields**. A field is marked in a similar fashion to markdown links but with square brackets.

```md
[Last Name][lastName]
```

This defines a field called `lastName` using the label `Last Name` at that position. The field name must be a valid javascript identifier.

To define the type of a field, a separate line with a syntax similar to markdown hrefs is used. This line should reference the field and provide additional information, like the field's type.

The default type of a field is `text` and for simple fields this is all that is needed. The two fields definitions are actually called placeholders and the type and other data of the field is implied. 

```md
[Number of calls][callCount]

[callCount]: number
```

Here we define `callCount` to be of type `number`.

It does not matter where the field type info appears but it makes sense to keep the field type definition close to the placeholder it is used in. Usually at the end of the corresponding section or paragraph if the section is long. This is similar to how urls are referenced in markdown.

> Note `Number of calls` is the label, `callCount` the field name and this can be referenced multiple times in a form.

Multiple fields can also be put next to each other in one line.

```md
[First Name][firstName] [Last Name][lastName]
```

## Sections

As with any markdown, it supports headers which are used to seperate a form into sections. The top section is called the root section. By default the root section contains all other sections, but if the top content is already a header and everything else is beneath that header level it is the root section.

```md
My content

[Name][name]

# My first subsection

Some more content

# My second subsection

```

```md
# My top header

## My subsection 1

### My subsection 3
```

When a document uses multiple top-level sections they are usually rendered as tabs by the form layout. Any content before the first section is part of the header.

## Types

Fields can have different types. Some examples:

```md
[age]: number
[birthdate]: date
[startedAt]: datetime
[address]: address
[preferredHand]: select
  - unknown
  - left
  - right
  - both
[address]: switch
...
```

### switch

The `switch` type represents a boolean value. Simply `true` or `false`. It is often used to keep track of conditional values.

### select

The select type represents a simple drop down box. It has a few different options depending on what is needed.

```md
[preferredHand]: 
  - 
  - left
  - right
  - both
```

The field shows and stores either an empty string (default because its the first value) or the strings 'left', 'right', or 'both'.

It is possible to store and display different values:

```md
[preferredHand]: 
  - 
  - left: Left
  - right: Right
  - both: Unknown
```

Where the value stored is e.g `left`, but the UI would show it as `Left`.

The initial value can also have a displayed text even if the value it an empty string in this example.

```md
[preferredHand]: 
  - : Unknown 
  - left: Left
  - right: Right
  - both: Unknown
```

## Help

![](/images/feature-help.png){ width=256 }

A field can be associated with some text which is usually displayed via a tooltip. This is done by appending a double quoted text behind the type.

```md
[age]: number "The contact's age at the time of the call"
```

It is also possible to provide extended help and other sections in markdown format.

```md
[age]: number 
  help:
  ---
  This is some *help* text in markdown format

[flightReadiness]: select
  - DEFAULT: Nothing selected
  - GREEN: Green Contact level
  - AMBER: Free to fly anything

  help:
  ---
  This is some *help* text in markdown format
```

## Tags

![](/images/feature-tags.png){ width=256 }

A field can also be tagged with various metadata.

```md
[field with validators][fieldWithValidators]
[fieldWithValidators]: <required min-length=10>

[Sensitive Value][sensitiveField]
[sensitiveField]: <optional sensitive expiry=5-years classification=dsgvo-5> 
```

> If a field is defined twice the last one overrides the initial value

::: info
Every `tag=value` segment inside the `<...>` is available later under the placeholder and field meta information. When a value is specified it is applied as a string. When no value is specified (e.g `<optional>`) it is treated as a flag and its value is true. There is no automatic conversion from `tag=false` to boolean or anything. If the tag is unwanted remove it or handle it as a string. 
:::

There are some well known tags that have some built-in meaning.

|Tag Name|Description|
|-|-|
|optional|Marks a field as optional. This means this field can be set to undefined|
|required|Marks a field as required which is used by validation policies|
|min-length|Used by the validation policies|
|max-length|Used by validation policies|

## Validation

![](/images/feature-validate.png){ width=256 }

From a data model perspective, it usually makes sense to allow all fields to be empty when working with forms, as this increases the flexibility of workflows a lot. When important information is required before further processing either manually or by business rules any validation policies can be verified before allowing further processing.

Using the `required` tag (See [above](#tags)) 

## Styling Directives

![](/images/feature-designer.png){ width=256 }

Fields and sections can have certain usability and styling rules associated with it. This is done with curly braces in front of the label. 

### Showing and hiding fields and sections

In some scenarios, it might be necessary to show or hide fields or entire sections within a form. There are two primary mechanisms for achieving this:

1) Show or hide values in the UI
2) Set data fields to `undefined`

### \{show if ...\} | \{hidden if ...\}

The styling directive placed before a field's placeholder or field definition can be configured to show or hide a field as needed.

```md
# My form

[Shipping Address][shipping]

[Different Billing Address][differentBillingAddress]
[differentBillingAddress]: Switch

{show if differentBillingAddress}[Billing Address][billing]
```

When `differentBillingAddress` is false, the user-interface control for the billing address will be hidden. It's important to note that hiding the control will not reset any of the values inside it. In most use cases, this approach is sufficient, but any code relying on this information will also need to consider the status of the switch field.

### Using `undefined`

As an alternative, a field or an entire section of fields can be marked as completely unavailable by setting their values to `undefined`. Fields with `undefined` values will not be rendered, and sections containing only `undefined` fields (including all subsections) will not be displayed either.

> Note: Undefined fields are **only** allowed when the form of that document (the exact version)
> explicitly permits them through the `allowUndefinedFields` setting.

When a form includes undefined fields, it's important to ensure that all formulas can handle this situation. In particular, expressions like `{show if myRefField.id}` could fail if not properly handled. To address this, use the `?` syntax: `{show if myRefField?.id}`. This approach allows the form designer to decide how to manage these cases.

You can also additionally tag those fields as optional. The default form rendering layout will use that information to allow users to show and hide such sections.

## Rendering

Forms are rendered by the `ae-form` directive. The rendering process is explained on a conceptual level [here](../020%20-%20Rendering/010-forms.md).
-----
Page Url: src/concepts/110 - Use Cases/050-number-sequences.md
-----
# Sequences of Numbers

An often required feature when writing a business application, is using a
reliable gap-free sequence of numbers in order to give documents
sequential numbers. The most common use case is bill numbers.

::: warning
This is NOT yet implemented and this design is not yet finalized. Progress is tracked in [#65](https://dev.curasystems.com/aeppic/aeppic-full/-/issues/65)
:::

# Implementation

The way this is done is having a well-known document representing the number set,
which tracks the highest number currently used and incrementing it on use.

Number document

```json
{
  ...
  "data": {
    ...
    "highestBillingNumber": 0
  }
}
```

Bill document:

```json
{
  ...
  "data": {
    ...
    "billNo": ""
  }
}
```

## Code

The field `highestBillingNumber` is used like a counter, but it is important to understand
that it isn't a normal counter which could just be reliable and concurrently incremented
using a concurrent increment function. In order to ensure reliable gap-free numbers the following code will work IF used in a sequentially serialized business rule. 

**Prerequisites**

- Business Rule configured to execute in series on all incoming bills that require a number
- The billing counter document cannot be changed by any other code

**Business Rule Action Code**

```javascript
const numberRangeDocument = await Aeppic.edit(Refs.BillingCounterDocument)
numberRangeDocument.increment('highestBillingNumber') 
Aeppic.save(numberRangeDocument) 

const newBill = await Aeppic.new({ form: Forms.Bill, parent: Refs.BillingFolder })

// ONLY DO THIS IF RUN IN A SEQUENTIAL BUSINESS RULE etc.
const nextBillNumber = numberRangeDocument.data.highestBillingNumber 
newBill.data.billNo = `BILL_${nextBillNumber.toString().padStart(10,'0')}`

Aeppic.save(newBill) 
```

**Why does it work ?**

By default business rule are guaranteed to execute within a temporarily consistent view of the read model with respect to changes made by previous iterations of the same business rule.

So if a business rule executes and changes a document, the next call to the same business rule can see that change reflected. So if a counter is increased (safely via `increment`)
and the next execution reads the same document the previous increment will be handled. It is
possible though that another source also changed that document in the meantime. Thus the field
might not be only incremented. To generate safe sequence of numbers therefore the developer
also needs to ensure that nobody changes the document otherwise. One way to do that is to lock the document and give write or even read access only to the business rule.

# Related code examples

## Wrong way

> NOTE: This pattern below CANNOT be used to generate sequential numbers. See above for that.

```javascript
const numberRangeDocument = await Aeppic.edit(Refs.BillingCounterDocument)
numberRangeDocument.highestBillingNumber ++ // THIS DOES NOT WORK RELIABLY, SEE BELOW FOR AN EXPLANATION
Aeppic.save(numberRangeDocument) 

const newBill = await Aeppic.new({ form: Forms.Bill, parent: Refs.BillingFolder })

const nextBillNumber = numberRangeDocument.data.highestBillingNumber
newBill.data.billNo = `BILL_${nextBillNumber.toString().padStart(10,'0')}`
// fill out rest of the bill (without a number)

Aeppic.save(newBill) 
```

The problem with this example is that `numberRangeDocument.highestBillingNumber ++ ` implicitly reads the field and then sets it afterwards. This loses information as to the intent behind the change.

It is equivalent to writing:

```js
const currentValue = numberRangeDocument.highestBillingNumber
numberRangeDocument.highestBillingNumber = currentValue + 1
```

The change committed during `Aeppic.save` is thus a `set` operation but the value of 
the field might have changed on the server in the meantime by concurrently running code.

## Counting

If we just wanted to count the number of bills created we could do the following:

::: warning
This pattern below CANNOT be used to generate sequential numbers. See above for that. It is an example
about implementing it wrongly.
:::

```javascript
const newBill = await Aeppic.new({ form: Forms.Bill, parent: Refs.BillingFolder })

// fill out a bill (without a number)

Aeppic.save(newBill)
Aeppic.Atomics.increment({ document: Refs.BillingCounterDocument, field: 'numberOfBillsCounter' })
```

This would work and be safe even if performed from a client, as both
changes (`Aeppic.save` and `Aeppic.Atomics.increment`) are executed in the same 
tick and thus automatically assigned the same Aeppic Save Context and therefore delivered
in the same server call. If the new bill gets saved, the billing counter will also be eventually
updated. If an Aeppic Save Context is manually assigned it can even be performed in one database
transaction and reflect at the same time, but that is usually not helpful.-----
Page Url: src/concepts/110 - Use Cases/README.md
-----
# A collection of typical scenarios

There are a number of typical use cases that need to be easy to be handled when
writing any application and business applications even more so.

The following list documents some of the ways these use cases should be implemented:

- [Sequences of Numbers](./050-number-sequences.md)

-----
Page Url: src/concepts/040 - Security/README.md
-----
# Security

All Documents and Commands are secured by a `Lock` and `Key` system. -----
Page Url: src/concepts/070 - Services/README.md
-----
# Services

Services extend the functionality of aeppic.

Examples:

* Thumbnail Service
* Email Service
* Document Converter Service (docx -> pdf)-----
Page Url: src/concepts/150 - HTML-Hierarchy/README.md
-----
------
# Understanding the aeppic default HTML Layout Hierarchy

When working with aeppic, you're not confined to the default HTML layout hierarchy. You have the freedom to configure a different root layout. However, understanding the default layouting hierarchy can be beneficial. It can help you make informed decisions about preserving certain elements, or understand the tradeoffs when making changes.

Web applications differ significantly from longform content, especially when it comes to dealing with overflow rules in HTML. While [CSS Grid](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Grid_Layout) provides a robust solution for creating complex layouts, it isn't always the perfect answer to all layout problems. The dynamic nature of aeppic - where all elements can be redefined at runtime - can further complicate things, especially when it comes to filling up remaining space.

Our primary goal is to create a classic layout with optional left and right bars, a top bar, and a central content area (which we refer to as `page`). These left and right bars should always remain in view, never scrolling away, while the central page area should take up the remaining height of the viewport.

Our current solution (which is not yet fully implemented with CSS Grid) relies on a global variable `var(--ae-max-top-content)`. This variable represents the exact pixel height of any content above the main page content.

::: info Recursive forms
The rendering pipeline below can contain other forms recursivley as content of a form. In that case the sub forms use the same layout templates, but do NOT limit the vertical height.
:::

## Key Considerations When Changing Layouts

When making changes to the layout, here are a few key points to consider:

1. **Maintain the Role of the `var(--ae-max-top-content)` Variable**: This variable is crucial for maintaining the layout's structure. It is used to calculate the maximum height of the main content area, ensuring it doesn't overflow the viewport. If you need to adjust the height of the top content, be sure to update this variable accordingly.

2. **Understanding Overflow Rules**: Overflow rules are essential for managing the display of content that exceeds its container's size. In our layout, the central page area is designed to be scrollable if the content overflows its height. If you're making changes to the structure, be mindful of how overflow rules may affect the display of content.

3. **Managing Dynamic Content**: In aeppic, all elements can be redefined at runtime, which can complicate layout management. When making changes, consider how dynamic content may affect the layout, especially when it comes to maintaining the fixed position of the left and right bars and managing the height of the central content area.

4. **Transitioning to CSS Grid**: CSS Grid offers a powerful and flexible solution for creating complex layouts. However, transitioning from a non-grid layout can require substantial changes to both the HTML structure and the CSS. Be sure to thoroughly test your layout after making any changes to ensure it behaves as expected.

In conclusion, the layout in aeppic is designed to provide a robust and flexible structure for a web application. It's designed to handle dynamic content, manage overflow effectively, and provide a consistent user experience. When making changes to this structure, consider how these changes may affect these key areas.

## CSS Structure Explanation

Leaving out some parts (right side bar etc) the basic layout on a conceptual level looks like this:

<iframe width="800" height="600" name="example-layout" sandbox="allow-scripts" src="/example-layout-1/index.html"/>

Open the layout in a new window <a href="/example-layout-1/index.html" target="_blank">here</a>

> An alternate example with a few more responsive tweaks is <a href="/example-layout-2/index.html" target="_blank">here</a> 

## Diagram 

```mermaid
graph TD
A[.main] --> B[.navbar]
A --> C[.center]
C --> D[.toolbar]
C --> E[.page]
E --> G[.ae-form.form-layout]
G --> H[.form-sidebar]
G --> J[.form-section-root-layout]
J --> K[.form-paragraphs-before-sections]
J --> L[.form-tabs-container]
L --> M[.form-section-tabs]
J --> N[.form-tab-content]
N --> O[.header]
N --> P[.spacer]
N --> Q[.footer]
```

## Classes in the example

### 1. Root Styles

`html`, `body`, and `div` are given basic styles to reset margins, padding, and to use border-box sizing.

### 2. Main Layout

- `.main`: It's a flex container with 100vh height and contains all elements.

- `.navbar`: A fixed-width navigation bar.

- `.center`: This flex item grows to take up all remaining horizontal space in the `.main`.

### 3. Toolbar

- `.toolbar`: A toolbar with a fixed height set by a CSS variable.

### 4. Page and ae-form

- `.page`: Container for the `.ae-form`.

- `.ae-form`: Container for the `.form-main-layout`. Renders in place no extra DOM element

### 5. Form Layout

- `.form-layout`: A grid container with two columns in our example. The first column is for the sidebar, and the second column grows to take up the remaining space to show the form.

- `.form-sidebar`: An optional fixed-size sidebar for a form .

### 6. Form Section Root Layout

- `.form-section-root-layout`: A flex container set to stack its children in a column. Its children have a minimum height of 0 to allow them to shrink as needed. When using tabs and the form isn't nested in another form the last element (the tab content) is set to `flex:1` and use the remaining space.

### 7. Form Before Sections Content

- `.form-paragraphs-before-sections`: A container for all content of the section before any subsections are defined.
- 
- `.form-tabs-container`: A grid container for the content to limit the tabs from overflowing.

- `.form-section-tabs`: A container for the form tabs representing the level 1 sub sections. It can scroll horizontally if the tabs overflow its width. The container itself is in a grid to ensure proper space usage.

### 8 Form Tab Content

This part is most often another section, but in this example we use:

- `.form-tab-content`: The container for the form content. It is set to grow and take up the remaining space in the `.form-section-root-layout`. It has a vertical scrollbar if the content overflows its height.

- `.header`: Headers within the form content. They are positioned as sticky at the top of their parent.

- `.spacer`: A spacer between headers, with a fixed height.

- `.footer`: The footer of the form content.
-----
Page Url: src/concepts/050 - Preferences/README.md
-----
# Overview

::: warning CAUTION
Any preferences lookup is cached immediately (!), unless `options.noCache=true`
:::

Preferences are used to retrieve settings by an arbitrary key. There are **fallback** and (document) **local** preferences. These are called `global` preferences.

Also, a single **user** may have an override to a single setting of a preference. Those are called `user` preferences.


A single fallback location of System Preferences (f.id: 50c9b51e-f477-48c0-94a9-b1445b287fd7) can be used to provide fallback preferences.
Local preferences are looked up to the nearest ancestor with a matching preference child.


## Active preferences

A preference needs to have the field `activeSettings` or `defaultUserSettings` defined or it will be ignored.

## User preferences

With each preference call, global and user preferences are looked up. A user setting takes precedence over a global setting, but only if the field has some value.

User preferences are always belonging to a single user (via `created.by`).

## Examples

```javascript
// fallback
const isEnabled = await Aeppic.Preferences.getSetting('some-namespace:my-preference:enabled')


// local, with possible fallback
const document = await Aeppic.get('some-uuid')
const isEnabled = await Aeppic.Preferences.getSettingForDocument('some-namespace:my-preference:enabled', document)
```


# Notes

Namespaces are required currently.-----
Page Url: src/concepts/060 - Packages/README.md
-----
# Intro to Packages

Packages in aeppic are a way to extend functionality. They are a standardized method of exposing javascript code, vue components, or assets to an aeppic web application.

::: info Why package ?
Dynamic code can be a security and stability risk. All methods of dynamic code are thus to be handled via linking it to an aeppic document which includes versioning and accountability.

The only exception is via the [server configuration](../../reference/aeppic-server/005-server-configuration.md#spa) which allows to statically define html/css/script files of any origin, but is handled by the server administrator anyway.
:::

## aeppic.json

Each package is a zip file in a `package` document. The zip includes an `aeppic.json` describing the content of the package. Once you have an `aeppic.json` (See below) it can be used to create the zip with  `ae-pkg pack . -o releases` which is a command exposed by `@aeppic/package`. That is the reason why the **package.json** of packages usually have a release script which does exactly that.

The `aeppic.json` used in the `workflow` package:

::: code-group
<<<@ ../../../packages/workflow/aeppic.json
:::

This is a very minimal example which defines an [ESM](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Modules) javascript package.

It exposes a single javascript module file to be loaded and all it's exports are made available by aeppic under the global `packages` variable inside e.g a controller.

The export endpoint `"."` represents the default endpoint to import. 

### Components

When the export also contains vue components these can be named under `components` which makes them available to Designs and Layouts.

```json
{
  ...
 "components": [
    "MonacoEditor"
  ],
  ...
}
```

A vue component that requires access to `Aeppic` can use `inject` with **getAeppicContent** which allows access to 
Aeppic via `this.getAeppicContext('<name-of-component-type-getting-context>' /*, this.document - optional aeppic document to link to */)`


```js
export default {
  template: '<div>GRAPH</div>',
  ...
  inject: ['getAeppicContext'],
  ...
  data() {
    const Aeppic = this.getAeppicContext('my-component' /*, this.document */) 
    ...
  }
}
```

### Assets and Stylesheets

When the components or main file require other assets these can be listed in the the assets or stylesheets sections. Stylesheets get loaded automatically.

```json
{ 
  ...
  "assets": [
    "./dist",
  ],
  "styles": [
    "./dist/stylesheet.css"
  ]
  ...
}
```-----
Page Url: src/concepts/README.md
-----
# Basic Concepts

**aeppic** is based on a few key concepts to achieve the architecture goals. There are only a few and rather simple concepts which should be known to write good apps.

